@@ -86,6 +86,7 @@ class FatalStartupError(Exception):
 def init_platform(
     basedir,
     configfile,
+    overlays=None,
     use_logging_file=True,
     logging_file=None,
     logging_config=None,
@@ -117,7 +118,7 @@ def init_platform(
         after_preinit_logging(**kwargs)
 
     try:
-        settings = init_settings(basedir, configfile)
+        settings = init_settings(basedir, configfile, overlays=overlays)
     except Exception as ex:
         raise FatalStartupError("Could not initialize settings manager", cause=ex)
     kwargs["settings"] = settings
@@ -220,13 +221,15 @@ def init_platform(
     )
 
 
-def init_settings(basedir, configfile):
+def init_settings(basedir, configfile, overlays=None):
     """Inits the settings instance based on basedir and configfile to use."""
 
     from octoprint.settings import InvalidSettings, settings
 
     try:
-        return settings(init=True, basedir=basedir, configfile=configfile)
+        return settings(
+            init=True, basedir=basedir, configfile=configfile, overlays=overlays
+        )
     except InvalidSettings as e:
         raise FatalStartupError(str(e))
 
@@ -284,11 +287,22 @@ def init_logging(
 
     # default logging configuration
     if default_config is None:
+        simple_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
         default_config = {
             "version": 1,
             "formatters": {
-                "simple": {
-                    "format": "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+                "simple": {"format": simple_format},
+                "colored": {
+                    "()": "colorlog.ColoredFormatter",
+                    "format": "%(log_color)s" + simple_format + "%(reset)s",
+                    "reset": True,
+                    "log_colors": {
+                        "DEBUG": "cyan",
+                        "INFO": "white",
+                        "WARNING": "yellow",
+                        "ERROR": "red",
+                        "CRITICAL": "bold_red",
+                    },
                 },
                 "serial": {"format": "%(asctime)s - %(message)s"},
                 "timings": {"format": "%(asctime)s - %(message)s"},
@@ -298,7 +312,7 @@ def init_logging(
                 "console": {
                     "class": "octoprint.logging.handlers.OctoPrintStreamHandler",
                     "level": "DEBUG",
-                    "formatter": "simple",
+                    "formatter": "colored",
                     "stream": "ext://sys.stdout",
                 },
                 "file": {
@@ -554,6 +568,8 @@ def set_logging_config(config, debug, verbosity, uncaught_logger, uncaught_handl
     # configure logging globally
     import logging.config as logconfig
 
+    from octoprint.logging.filters import TornadoAccessFilter
+
     logconfig.dictConfig(config)
 
     # make sure we log any warnings
@@ -582,6 +598,9 @@ def set_logging_config(config, debug, verbosity, uncaught_logger, uncaught_handl
         uncaught_handler = exception_logger
     sys.excepthook = uncaught_handler
 
+    tornado_logger = log.getLogger("tornado.access")
+    tornado_logger.addFilter(TornadoAccessFilter())
+
     return logger
 
 
@@ -619,16 +638,17 @@ def init_pluginsystem(
         )
 
     plugin_validators = []
+
     if safe_mode:
 
-        def validator(phase, plugin_info):
+        def safe_mode_validator(phase, plugin_info):
             if phase in ("before_import", "before_load", "before_enable"):
                 plugin_info.safe_mode_victim = not plugin_info.bundled
                 if not plugin_info.bundled:
                     return False
             return True
 
-        plugin_validators.append(validator)
+        plugin_validators.append(safe_mode_validator)
 
     compatibility_ignored_list = settings.get(["plugins", "_forcedCompatible"])
 
@@ -725,7 +745,7 @@ def get_plugin_blacklist(settings, connectivity_checker=None):
     import yaml
 
     from octoprint.util import bom_aware_open
-    from octoprint.util.version import is_octoprint_compatible
+    from octoprint.util.version import is_octoprint_compatible, is_python_compatible
 
     logger = log.getLogger(__name__ + ".startup")
 
@@ -756,13 +776,19 @@ def get_plugin_blacklist(settings, connectivity_checker=None):
             ):
                 continue
 
-            if "version" in entry:
+            if "pythonversions" in entry and not is_python_compatible(
+                *entry["pythonversions"]
+            ):
+                continue
+
+            if "pluginversions" in entry:
                 logger.debug(
-                    "Blacklisted plugin: {}, version: {}".format(
-                        entry["plugin"], entry["version"]
+                    "Blacklisted plugin: {}, versions: {}".format(
+                        entry["plugin"], ", ".join(entry["pluginversions"])
                     )
                 )
-                result.append((entry["plugin"], entry["version"]))
+                for version in entry["pluginversions"]:
+                    result.append((entry["plugin"], version))
             elif "versions" in entry:
                 logger.debug(
                     "Blacklisted plugin: {}, versions: {}".format(
@@ -770,7 +796,7 @@ def get_plugin_blacklist(settings, connectivity_checker=None):
                     )
                 )
                 for version in entry["versions"]:
-                    result.append((entry["plugin"], version))
+                    result.append((entry["plugin"], "=={}".format(version)))
             else:
                 logger.debug("Blacklisted plugin: {}".format(entry["plugin"]))
                 result.append(entry["plugin"])
