@@ -6,17 +6,28 @@
 
 var gcode;
 var firstReport;
-var toolOffsets = [
-    {x: 0, y: 0}
-];
+var toolOffsets = [{x: 0, y: 0}];
 var g90InfluencesExtruder = false;
 var z_heights = {};
 var model = [];
 var max = {x: undefined, y: undefined, z: undefined};
 var min = {x: undefined, y: undefined, z: undefined};
-var boundingBox = {minX: undefined, maxX: undefined, minY: undefined, maxY: undefined, minZ: undefined, maxZ: undefined};
+var boundingBox = {
+    minX: undefined,
+    maxX: undefined,
+    minY: undefined,
+    maxY: undefined,
+    minZ: undefined,
+    maxZ: undefined
+};
 var modelSize = {x: undefined, y: undefined, z: undefined};
-var bed = {x: undefined, y: undefined, r: undefined, circular: false, centeredOrigin: false};
+var bed = {
+    x: undefined,
+    y: undefined,
+    r: undefined,
+    circular: false,
+    centeredOrigin: false
+};
 var ignoreOutsideBed = false;
 var filamentByLayer = {};
 var totalFilament = [0];
@@ -29,8 +40,8 @@ var speedsByLayer = {extrude: {}, retract: {}, move: {}};
 
 var sendLayerToParent = function (layerNum, z, progress) {
     self.postMessage({
-        "cmd": "returnLayer",
-        "msg": {
+        cmd: "returnLayer",
+        msg: {
             cmds: model[layerNum],
             layerNum: layerNum,
             zHeightObject: {zValue: z, layer: z_heights[z]},
@@ -50,8 +61,8 @@ var sendMultiLayerToParent = function (layerNum, z, progress) {
     }
 
     self.postMessage({
-        "cmd": "returnMultiLayer",
-        "msg": {
+        cmd: "returnMultiLayer",
+        msg: {
             model: tmpModel,
             layerNum: layerNum,
             zHeightObject: {zValue: z, layer: tmpZHeight},
@@ -61,11 +72,10 @@ var sendMultiLayerToParent = function (layerNum, z, progress) {
     });
 };
 
-
 var sendSizeProgress = function (progress) {
     self.postMessage({
-        "cmd": "analyzeProgress",
-        "msg": {
+        cmd: "analyzeProgress",
+        msg: {
             progress: progress,
             printTime: printTime
         }
@@ -74,8 +84,8 @@ var sendSizeProgress = function (progress) {
 
 var sendAnalyzeDone = function () {
     self.postMessage({
-        "cmd": "analyzeDone",
-        "msg": {
+        cmd: "analyzeDone",
+        msg: {
             max: max,
             min: min,
             modelSize: modelSize,
@@ -101,7 +111,7 @@ var purgeLayers = function () {
             purge = true;
         } else {
             for (var j = 0; j < model[i].length; j++) {
-                if (model[i][j].extrude)purge = false;
+                if (model[i][j].extrude) purge = false;
             }
         }
         if (!purge) {
@@ -110,7 +120,7 @@ var purgeLayers = function () {
     }
 };
 
-var bedBounds = function() {
+var bedBounds = function () {
     if (!bed) {
         return {xMin: 0, xMax: 0, yMin: 0, yMax: 0};
     }
@@ -119,30 +129,36 @@ var bedBounds = function() {
     if (bed.circular) {
         result = {xMin: -bed.r, xMax: bed.r, yMin: -bed.r, yMax: bed.r};
     } else if (bed.centeredOrigin) {
-        result = {xMin: -bed.x/2.0, xMax: bed.x/2.0, yMin: -bed.y/2.0, yMax: bed.y/2.0};
+        result = {
+            xMin: -bed.x / 2.0,
+            xMax: bed.x / 2.0,
+            yMin: -bed.y / 2.0,
+            yMax: bed.y / 2.0
+        };
     } else {
         result = {xMin: 0, xMax: bed.x, yMin: 0, yMax: bed.y};
     }
     return result;
 };
 
-var withinBedBounds = function(x, y, bedBounds) {
+var withinBedBounds = function (x, y, bedBounds) {
     if (!ignoreOutsideBed) return true;
 
     var result = true;
 
     if (x !== undefined && !isNaN(x)) {
-        result = result && (x >= bedBounds.xMin && x <= bedBounds.xMax);
+        result = result && x >= bedBounds.xMin && x <= bedBounds.xMax;
     }
     if (y !== undefined && !isNaN(y)) {
-        result = result && (y >= bedBounds.yMin && y <= bedBounds.yMax);
+        result = result && y >= bedBounds.yMin && y <= bedBounds.yMax;
     }
 
     return result;
 };
 
 var analyzeModel = function () {
-    var tmp1 = 0, tmp2 = 0;
+    var tmp1 = 0,
+        tmp2 = 0;
     var speedIndex = 0;
     var type;
     var printTimeAdd = 0;
@@ -158,38 +174,55 @@ var analyzeModel = function () {
 
             var retract = cmds[j].retract && cmds[j].retract > 0;
             var extrude = cmds[j].extrude && cmds[j].extrude > 0 && !retract;
-            var move = cmds[j].x !== undefined || cmds[j].y !== undefined || cmds[j].z !== undefined;
+            var move =
+                cmds[j].x !== undefined ||
+                cmds[j].y !== undefined ||
+                cmds[j].z !== undefined;
 
             var prevInBounds = withinBedBounds(cmds[j].prevX, cmds[j].prevY, bounds);
-            var inBounds = withinBedBounds(cmds[j].x === undefined ? cmds[j].prevX : cmds[j].x,
-                                           cmds[j].y === undefined ? cmds[j].prevY : cmds[j].y,
-                                           bounds);
+            var inBounds = withinBedBounds(
+                cmds[j].x === undefined ? cmds[j].prevX : cmds[j].x,
+                cmds[j].y === undefined ? cmds[j].prevY : cmds[j].y,
+                bounds
+            );
 
-            var recordX = function(x, inBounds) {
+            var recordX = function (x, inBounds) {
                 if (x === undefined || isNaN(x)) return;
 
                 max.x = max.x !== undefined ? Math.max(max.x, x) : x;
                 min.x = min.x !== undefined ? Math.min(min.x, x) : x;
 
                 if (inBounds) {
-                    boundingBox.minX = boundingBox.minX !== undefined ? Math.min(boundingBox.minX, x): x;
-                    boundingBox.maxX = boundingBox.maxX !== undefined ? Math.max(boundingBox.maxX, x): x;
+                    boundingBox.minX =
+                        boundingBox.minX !== undefined
+                            ? Math.min(boundingBox.minX, x)
+                            : x;
+                    boundingBox.maxX =
+                        boundingBox.maxX !== undefined
+                            ? Math.max(boundingBox.maxX, x)
+                            : x;
                 }
             };
 
-            var recordY = function(y, inBounds) {
+            var recordY = function (y, inBounds) {
                 if (y === undefined || isNaN(y)) return;
 
                 max.y = max.y !== undefined ? Math.max(max.y, y) : y;
                 min.y = min.y !== undefined ? Math.min(min.y, y) : y;
 
                 if (inBounds) {
-                    boundingBox.minY = boundingBox.minY !== undefined ? Math.min(boundingBox.minY, y): y;
-                    boundingBox.maxY = boundingBox.maxY !== undefined ? Math.max(boundingBox.maxY, y): y;
+                    boundingBox.minY =
+                        boundingBox.minY !== undefined
+                            ? Math.min(boundingBox.minY, y)
+                            : y;
+                    boundingBox.maxY =
+                        boundingBox.maxY !== undefined
+                            ? Math.max(boundingBox.maxY, y)
+                            : y;
                 }
             };
 
-            var recordZ = function(z) {
+            var recordZ = function (z) {
                 if (z === undefined || isNaN(z)) return;
 
                 max.z = max.z !== undefined ? Math.max(max.z, z) : z;
@@ -210,20 +243,27 @@ var analyzeModel = function () {
 
             if (!totalFilament[tool]) totalFilament[tool] = 0;
             if (!filamentByLayer[cmds[j].prevZ]) filamentByLayer[cmds[j].prevZ] = [0];
-            if (!filamentByLayer[cmds[j].prevZ][tool]) filamentByLayer[cmds[j].prevZ][tool] = 0;
+            if (!filamentByLayer[cmds[j].prevZ][tool])
+                filamentByLayer[cmds[j].prevZ][tool] = 0;
             if (cmds[j].extrusion) {
                 totalFilament[tool] += cmds[j].extrusion;
                 filamentByLayer[cmds[j].prevZ][tool] += cmds[j].extrusion;
             }
 
-            if (cmds[j].x !== undefined && !isNaN(cmds[j].x)
-                    && cmds[j].y !== undefined && !isNaN(cmds[j].y)) {
+            if (
+                cmds[j].x !== undefined &&
+                !isNaN(cmds[j].x) &&
+                cmds[j].y !== undefined &&
+                !isNaN(cmds[j].y)
+            ) {
                 var diffX = cmds[j].x - cmds[j].prevX;
                 var diffY = cmds[j].y - cmds[j].prevY;
                 if (move) {
-                    printTimeAdd = Math.sqrt(diffX * diffX + diffY * diffY) / (cmds[j].speed / 60);
+                    printTimeAdd =
+                        Math.sqrt(diffX * diffX + diffY * diffY) / (cmds[j].speed / 60);
                 } else if (extrude) {
-                    tmp1 = Math.sqrt(diffX * diffX + diffY * diffY) / (cmds[j].speed / 60);
+                    tmp1 =
+                        Math.sqrt(diffX * diffX + diffY * diffY) / (cmds[j].speed / 60);
                     tmp2 = Math.abs(cmds[j].extrusion / (cmds[j].speed / 60));
                     printTimeAdd = Math.max(tmp1, tmp2);
                 } else if (retract) {
@@ -240,14 +280,14 @@ var analyzeModel = function () {
             printTimeByLayer[cmds[j].prevZ] += printTimeAdd;
 
             if (cmds[j].extrude && cmds[j].retract === 0) {
-                type = 'extrude';
+                type = "extrude";
             } else if (cmds[j].retract !== 0) {
-                type = 'retract';
+                type = "retract";
             } else if (!cmds[j].extrude && cmds[j].retract === 0) {
-                type = 'move';
+                type = "move";
             } else {
-                self.postMessage({cmd: 'unknown type of move'});
-                type = 'unknown';
+                self.postMessage({cmd: "unknown type of move"});
+                type = "unknown";
             }
 
             speedIndex = speeds[type].indexOf(cmds[j].speed);
@@ -255,16 +295,14 @@ var analyzeModel = function () {
                 speeds[type].push(cmds[j].speed);
                 speedIndex = speeds[type].indexOf(cmds[j].speed);
             }
-            if (typeof(speedsByLayer[type][cmds[j].prevZ]) === 'undefined') {
+            if (typeof speedsByLayer[type][cmds[j].prevZ] === "undefined") {
                 speedsByLayer[type][cmds[j].prevZ] = [];
             }
             if (speedsByLayer[type][cmds[j].prevZ].indexOf(cmds[j].speed) === -1) {
                 speedsByLayer[type][cmds[j].prevZ][speedIndex] = cmds[j].speed;
             }
-
         }
-        sendSizeProgress(i / model.length * 100);
-
+        sendSizeProgress((i / model.length) * 100);
     }
     purgeLayers();
 
@@ -285,11 +323,18 @@ var doParse = function () {
     var lastSend = 0;
 
     var layer = 0;
-    var x, y, z = 0;
+    var x,
+        y,
+        z = 0;
     var center_i, center_j, direction;
-    var prevX = 0, prevY = 0, prevZ = 0;
-    var f, lastF = 4000;
-    var extrude = false, relativeE = false, retract = 0;
+    var prevX = 0,
+        prevY = 0,
+        prevZ = 0;
+    var f,
+        lastF = 4000;
+    var extrude = false,
+        relativeE = false,
+        retract = 0;
     var relativeMode = false;
     var zLift = false;
     var zLiftZ = undefined;
@@ -339,8 +384,8 @@ var doParse = function () {
             args = line.split(/\s/);
 
             for (j = 0; j < args.length; j++) {
-                switch (argChar = args[j].charAt(0).toLowerCase()) {
-                    case 'x':
+                switch ((argChar = args[j].charAt(0).toLowerCase())) {
+                    case "x":
                         if (relativeMode) {
                             x = prevX + Number(args[j].slice(1)) + activeToolOffset.x;
                         } else {
@@ -349,7 +394,7 @@ var doParse = function () {
 
                         break;
 
-                    case 'y':
+                    case "y":
                         if (relativeMode) {
                             y = prevY + Number(args[j].slice(1)) + activeToolOffset.y;
                         } else {
@@ -358,7 +403,7 @@ var doParse = function () {
 
                         break;
 
-                    case 'z':
+                    case "z":
                         if (relativeMode) {
                             z = prevZ + Number(args[j].slice(1));
                         } else {
@@ -367,10 +412,10 @@ var doParse = function () {
 
                         break;
 
-                    case 'e':
-                    case 'a':
-                    case 'b':
-                    case 'c':
+                    case "e":
+                    case "a":
+                    case "b":
+                    case "c":
                         assumeNonDC = true;
                         numSlice = Number(args[j].slice(1));
 
@@ -379,7 +424,8 @@ var doParse = function () {
                             prev_extrude[tool][argChar] += numSlice;
                         } else {
                             // absolute extrusion positioning
-                            prev_extrude[tool]["abs"] = numSlice - prev_extrude[tool][argChar];
+                            prev_extrude[tool]["abs"] =
+                                numSlice - prev_extrude[tool][argChar];
                             prev_extrude[tool][argChar] = numSlice;
                         }
 
@@ -389,7 +435,10 @@ var doParse = function () {
                             retract = -1;
                         } else if (prev_extrude[tool]["abs"] === 0) {
                             retract = 0;
-                        } else if (prev_extrude[tool]["abs"] > 0 && prev_retract[tool] < 0) {
+                        } else if (
+                            prev_extrude[tool]["abs"] > 0 &&
+                            prev_retract[tool] < 0
+                        ) {
                             prev_retract[tool] = 0;
                             retract = 1;
                         } else {
@@ -398,31 +447,36 @@ var doParse = function () {
 
                         break;
 
-                    case 'f':
+                    case "f":
                         numSlice = parseFloat(args[j].slice(1));
                         lastF = numSlice;
                         break;
-                    case 'i':
+                    case "i":
                         center_i = Number(args[j].slice(1));
                         break;
-                    case 'j':
+                    case "j":
                         center_j = Number(args[j].slice(1));
                         break;
-                    case 'g':
-                        if(args[j].charAt(1).toLowerCase() === '2')
-                            direction=-1;
-                        if(args[j].charAt(1).toLowerCase() === '3')
-                            direction=1;
+                    case "g":
+                        if (args[j].charAt(1).toLowerCase() === "2") direction = -1;
+                        if (args[j].charAt(1).toLowerCase() === "3") direction = 1;
                         break;
                 }
             }
 
             if (dcExtrude && !assumeNonDC) {
                 extrude = true;
-                prev_extrude[tool]["abs"] = Math.sqrt((prevX - x) * (prevX - x) + (prevY - y) * (prevY - y));
+                prev_extrude[tool]["abs"] = Math.sqrt(
+                    (prevX - x) * (prevX - x) + (prevY - y) * (prevY - y)
+                );
             }
 
-            if (typeof(x) !== 'undefined' || typeof(y) !== 'undefined' || typeof(z) !== 'undefined' || retract !== 0) {
+            if (
+                typeof x !== "undefined" ||
+                typeof y !== "undefined" ||
+                typeof z !== "undefined" ||
+                retract !== 0
+            ) {
                 addToModel = true;
                 move = true;
             }
@@ -460,24 +514,24 @@ var doParse = function () {
                     prev_extrude[tool]["b"] = 0;
                     prev_extrude[tool]["c"] = 0;
                 } else {
-                    switch (argChar = args[j].charAt(0).toLowerCase()) {
-                        case 'x':
+                    switch ((argChar = args[j].charAt(0).toLowerCase())) {
+                        case "x":
                             x = Number(args[j].slice(1)) + activeToolOffset.x;
                             break;
 
-                        case 'y':
+                        case "y":
                             y = Number(args[j].slice(1)) + activeToolOffset.y;
                             break;
 
-                        case 'z':
+                        case "z":
                             z = Number(args[j].slice(1));
                             prevZ = z;
                             break;
 
-                        case 'e':
-                        case 'a':
-                        case 'b':
-                        case 'c':
+                        case "e":
+                        case "a":
+                        case "b":
+                        case "c":
                             numSlice = Number(args[j].slice(1));
                             if (relativeMode || relativeE) {
                                 prev_extrude[tool][argChar] = numSlice;
@@ -489,11 +543,14 @@ var doParse = function () {
                 }
             }
 
-            if (typeof(x) !== 'undefined' || typeof(y) !== 'undefined' || typeof(z) !== 'undefined') {
+            if (
+                typeof x !== "undefined" ||
+                typeof y !== "undefined" ||
+                typeof z !== "undefined"
+            ) {
                 addToModel = true;
                 move = false;
             }
-
         } else if (/^(?:G28)(\.\d+)?/i.test(line)) {
             args = line.split(/\s/);
 
@@ -504,14 +561,14 @@ var doParse = function () {
                 z = 0;
             } else {
                 for (j = 0; j < args.length; j++) {
-                    switch (argChar = args[j].charAt(0).toLowerCase()) {
-                        case 'x':
+                    switch ((argChar = args[j].charAt(0).toLowerCase())) {
+                        case "x":
                             x = 0;
                             break;
-                        case 'y':
+                        case "y":
                             y = 0;
                             break;
-                        case 'z':
+                        case "z":
                             z = 0;
                             break;
                         default:
@@ -521,17 +578,23 @@ var doParse = function () {
             }
 
             // if it's the first layer and G28 was without z
-            if (layer === 0 && typeof(z) === 'undefined') {
+            if (layer === 0 && typeof z === "undefined") {
                 z = 0;
             }
 
-            if (typeof(x) !== 'undefined' || typeof(y) !== 'undefined' || typeof(z) !== 'undefined' || retract !== 0) {
+            if (
+                typeof x !== "undefined" ||
+                typeof y !== "undefined" ||
+                typeof z !== "undefined" ||
+                retract !== 0
+            ) {
                 addToModel = true;
                 move = true;
             }
         } else if (/^(?:T\d+)/i.test(line)) {
             tool = Number(line.split(/\s/)[0].slice(1));
-            if (!prev_extrude[tool]) prev_extrude[tool] = {a: 0, b: 0, c: 0, e: 0, abs: 0};
+            if (!prev_extrude[tool])
+                prev_extrude[tool] = {a: 0, b: 0, c: 0, e: 0, abs: 0};
             if (!prev_retract[tool]) prev_retract[tool] = 0;
 
             activeToolOffset = toolOffsets[tool];
@@ -540,7 +603,7 @@ var doParse = function () {
 
         // If move is on a new height and it's not extruding and
         // it's not currently already in a Z-lift, assume it's possibly a Z-lift
-        if (typeof(z) !== 'undefined' && z !== prevZ && !extrude && !zLift) {
+        if (typeof z !== "undefined" && z !== prevZ && !extrude && !zLift) {
             zLift = true;
             zLiftZ = prevZ;
         }
@@ -549,7 +612,7 @@ var doParse = function () {
             zLift = false;
         }
 
-        if (typeof(z) !== 'undefined' && z !== prevZ) {
+        if (typeof z !== "undefined" && z !== prevZ) {
             if (z_heights[z] !== undefined) {
                 layer = z_heights[z];
             } else {
@@ -560,7 +623,7 @@ var doParse = function () {
             sendLayer = layer;
             sendLayerZ = z;
             prevZ = z;
-        } else if (typeof(z) === 'undefined' && typeof(prevZ) !== 'undefined') {
+        } else if (typeof z === "undefined" && typeof prevZ !== "undefined") {
             if (z_heights.hasOwnProperty(prevZ)) {
                 layer = z_heights[prevZ];
             } else {
@@ -581,7 +644,10 @@ var doParse = function () {
                 extrude: extrude,
                 retract: retract,
                 noMove: !move,
-                extrusion: (extrude || retract) && prev_extrude[tool]["abs"] ? prev_extrude[tool]["abs"] : 0,
+                extrusion:
+                    (extrude || retract) && prev_extrude[tool]["abs"]
+                        ? prev_extrude[tool]["abs"]
+                        : 0,
                 prevX: prevX,
                 prevY: prevY,
                 prevZ: prevZ,
@@ -601,7 +667,10 @@ var doParse = function () {
                 // there's something to be checked in the Z-lift cache
                 if (prevZ === zLiftZ) {
                     zLiftMoves.forEach(function (zLiftMove) {
-                        model[zLiftMove.layer].splice(model[layer].indexOf(zLiftMove.command), 1);
+                        model[zLiftMove.layer].splice(
+                            model[layer].indexOf(zLiftMove.command),
+                            1
+                        );
                         model[z_heights[zLiftZ]].push(zLiftMove.command);
                     });
                 }
@@ -610,19 +679,22 @@ var doParse = function () {
                 zLiftZ = undefined;
             }
 
-
             model[layer].push(command);
         }
 
         if (move) {
-            if (typeof(x) !== 'undefined') prevX = x;
-            if (typeof(y) !== 'undefined') prevY = y;
+            if (typeof x !== "undefined") prevX = x;
+            if (typeof y !== "undefined") prevY = y;
         }
 
-        if (typeof(sendLayer) !== "undefined") {
+        if (typeof sendLayer !== "undefined") {
             if (i - lastSend > gcode.length * 0.02 && sendMultiLayer.length !== 0) {
                 lastSend = i;
-                sendMultiLayerToParent(sendMultiLayer, sendMultiLayerZ, i / gcode.length * 100);
+                sendMultiLayerToParent(
+                    sendMultiLayer,
+                    sendMultiLayerZ,
+                    (i / gcode.length) * 100
+                );
                 sendMultiLayer = [];
                 sendMultiLayerZ = [];
             }
@@ -632,10 +704,9 @@ var doParse = function () {
             sendLayerZ = undefined;
         }
     }
-    sendMultiLayerToParent(sendMultiLayer, sendMultiLayerZ, i / gcode.length * 100);
+    sendMultiLayerToParent(sendMultiLayer, sendMultiLayerZ, (i / gcode.length) * 100);
 };
 
-
 var parseGCode = function (message) {
     gcode = message.gcode;
     firstReport = message.options.firstReport;
@@ -644,12 +715,13 @@ var parseGCode = function (message) {
     bed = message.options.bed;
     ignoreOutsideBed = message.options.ignoreOutsideBed;
     g90InfluencesExtruder = message.options.g90InfluencesExtruder;
+    boundingBox.minZ = min.z = message.options.bedZ;
 
     doParse();
     gcode = [];
     self.postMessage({
-        "cmd": "returnModel",
-        "msg": {}
+        cmd: "returnModel",
+        msg: {}
     });
 };
 
@@ -664,7 +736,14 @@ var runAnalyze = function (message) {
     max = {x: undefined, y: undefined, z: undefined};
     min = {x: undefined, y: undefined, z: undefined};
     modelSize = {x: undefined, y: undefined, z: undefined};
-    boundingBox = {minX: undefined, maxX: undefined, minY: undefined, maxY: undefined, minZ: undefined, maxZ: undefined};
+    boundingBox = {
+        minX: undefined,
+        maxX: undefined,
+        minY: undefined,
+        maxY: undefined,
+        minZ: undefined,
+        maxZ: undefined
+    };
     filamentByLayer = {};
     totalFilament = 0;
     printTime = 0;
@@ -686,18 +765,17 @@ onmessage = function (e) {
     var data = e.data;
     // for some reason firefox doesn't garbage collect when something inside closures is deleted, so we delete and recreate whole object eaech time
     switch (data.cmd) {
-        case 'parseGCode':
+        case "parseGCode":
             parseGCode(data.msg);
             break;
-        case 'setOption':
+        case "setOption":
             setOption(data.msg);
             break;
-        case 'analyzeModel':
+        case "analyzeModel":
             runAnalyze(data.msg);
             break;
 
         default:
-            self.postMessage('Unknown command: ' + data.msg);
+            self.postMessage("Unknown command: " + data.msg);
     }
-
 };
