@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import datetime
+
 __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
@@ -34,14 +36,20 @@ from octoprint.server.util.flask import (
 )
 from octoprint.util import dict_merge, get_formatted_size, to_unicode
 from octoprint.util.pip import create_pip_caller
-from octoprint.util.version import get_comparable_version, get_python_version_string
+from octoprint.util.version import (
+    get_comparable_version,
+    get_python_version_string,
+    is_python_compatible,
+)
 
 from . import cli, exceptions, updaters, util, version_checks
 
-# OctoPi 0.15+
-MINIMUM_PYTHON = "2.7.9"
-MINIMUM_SETUPTOOLS = "39.0.1"
-MINIMUM_PIP = "9.0.3"
+# OctoPi 0.16+
+MINIMUM_PYTHON = "2.7.13"
+MINIMUM_SETUPTOOLS = "40.7.1"
+MINIMUM_PIP = "19.0.1"
+
+DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
 
 
 ##~~ Plugin
@@ -62,6 +70,8 @@ class SoftwareUpdatePlugin(
 
     OCTOPRINT_RESTART_TYPES = ("pip", "single_file_plugin")
 
+    VALID_RESTART_TYPES = ("octoprint", "environment")
+
     DATA_FORMAT_VERSION = "v4"
 
     CHECK_OVERLAY_KEY = "softwareupdate_check_overlay"
@@ -87,12 +97,17 @@ class SoftwareUpdatePlugin(
         self._overlay_cache_ttl = 0
         self._overlay_cache_timestamp = None
 
+        self._update_log = []
+        self._update_log_path = None
+        self._update_log_dirty = False
+        self._update_log_mutex = threading.RLock()
+
         self._environment_supported = True
         self._environment_versions = {}
         self._environment_ready = threading.Event()
 
         self._storage_sufficient = True
-        self.storage_info = {}
+        self._storage_info = {}
 
         self._console_logger = None
 
@@ -109,6 +124,11 @@ class SoftwareUpdatePlugin(
         )
         self._load_version_cache()
 
+        self._update_log_path = os.path.join(
+            self.get_plugin_data_folder(), "updatelog.yaml"
+        )
+        self._load_update_log()
+
         self._overlay_cache_ttl = self._settings.get_int(["check_overlay_ttl"]) * 60
 
         def refresh_checks(name, plugin):
@@ -147,6 +167,38 @@ class SoftwareUpdatePlugin(
             },
         ]
 
+    # Additional bundle contents
+
+    def get_additional_bundle_files(self, *args, **kwargs):
+        console_log = self._settings.get_plugin_logfile_path(postfix="console")
+
+        def updatelog():
+            if not self._update_log:
+                self._load_update_log()
+
+            result = []
+            for entry in self._update_log:
+                line = "{datetime} - {display_name} - {current_version} -> {target_version} - {success} - {text}".format(
+                    datetime=entry["datetime"].replace("T", " "),
+                    display_name=entry["display_name"],
+                    current_version=entry["current_version"],
+                    target_version=entry["target_version"],
+                    success="SUCCESS" if entry["success"] else "FAILED",
+                    text=entry["text"]
+                    + (
+                        " (Release notes: {})".format(entry["release_notes"])
+                        if entry["release_notes"]
+                        else ""
+                    ),
+                )
+                result.append(line)
+            return "\n".join(result)
+
+        return {
+            os.path.basename(console_log): console_log,
+            "plugin_softwareupdate_update.log": updatelog,
+        }
+
     def on_startup(self, host, port):
         console_logging_handler = logging.handlers.RotatingFileHandler(
             self._settings.get_plugin_logfile_path(postfix="console"),
@@ -372,23 +424,33 @@ class SoftwareUpdatePlugin(
             or self._overlay_cache_timestamp + self._overlay_cache_ttl < time.time()
         )
 
+    def _get_check_overlay(self, url):
+        self._logger.info("Fetching check overlays from {}".format(url))
+        try:
+            r = requests.get(url, timeout=3.1)
+            r.raise_for_status()
+            data = r.json()
+        except Exception as exc:
+            self._logger.error(
+                "Could not fetch check overlay from {}: {}".format(url, exc)
+            )
+            return {}
+        else:
+            return data
+
     def _get_check_overlays(self, force=False):
         if self._check_overlays_stale or force:
             if self._connectivity_checker.online:
-                # reload from URL
-                url = self._settings.get(["check_overlay_url"])
-                self._logger.info("Fetching check overlays from {}".format(url))
-                try:
-                    r = requests.get(url, timeout=3.1)
-                    r.raise_for_status()
-                    data = r.json()
-                except Exception as exc:
-                    self._logger.error(
-                        "Could not fetch check overlay from {}: {}".format(url, exc)
+                self._overlay_cache = self._get_check_overlay(
+                    self._settings.get(["check_overlay_url"])
+                )
+                if is_python_compatible("<3"):
+                    data = self._get_check_overlay(
+                        self._settings.get(["check_overlay_py2_url"])
+                    )
+                    self._overlay_cache = octoprint.util.dict_merge(
+                        self._overlay_cache, data
                     )
-                    self._overlay_cache = {}
-                else:
-                    self._overlay_cache = data
 
             else:
                 self._logger.info("Not fetching check overlays, we are offline")
@@ -486,6 +548,138 @@ class SoftwareUpdatePlugin(
             pass
         self._version_cache_dirty = True
 
+    def _load_update_log(self):
+        if not os.path.isfile(self._update_log_path):
+            return
+
+        import yaml
+
+        try:
+            with io.open(self._update_log_path, "rt", encoding="utf-8") as f:
+                data = yaml.safe_load(f)
+        except Exception:
+            self._logger.exception("Error while loading update log from disk")
+        else:
+            # clean up data older than updatelog_cutoff hours
+            before_cleanup = len(data)
+            cutoff = (
+                datetime.datetime.now()
+                - datetime.timedelta(minutes=self._settings.get_int(["updatelog_cutoff"]))
+            ).strftime(DATETIME_FORMAT)
+            data = sorted(
+                list(
+                    filter(
+                        lambda x: x.get("datetime") and x["datetime"] > cutoff,
+                        data,
+                    )
+                ),
+                key=lambda x: x["datetime"],
+            )
+            cleaned_up = len(data) - before_cleanup
+            if cleaned_up:
+                self._logger.info(
+                    "Cleaned up {} old update log entries".format(cleaned_up)
+                )
+
+            with self._update_log_mutex:
+                self._update_log = data
+                self._update_log_dirty = False
+
+            self._logger.info("Loaded update log from disk")
+
+    def _save_update_log(self):
+        import yaml
+
+        from octoprint.util import atomic_write
+
+        with self._update_log_mutex:
+            with atomic_write(
+                self._update_log_path, mode="wt", max_permissions=0o666
+            ) as file_obj:
+                yaml.safe_dump(
+                    sorted(self._update_log, key=lambda x: x["datetime"]),
+                    stream=file_obj,
+                    default_flow_style=False,
+                    indent=2,
+                    allow_unicode=True,
+                )
+                self._update_log_dirty = False
+
+        self._logger.info("Saved update log to disk")
+
+    def _add_update_log_entry(
+        self,
+        target,
+        current_version,
+        target_version,
+        success,
+        text,
+        display_name=None,
+        release_notes="",
+        save=True,
+    ):
+        if display_name is None:
+            display_name = target
+
+        with self._update_log_mutex:
+            self._update_log.append(
+                {
+                    "datetime": datetime.datetime.now().strftime(DATETIME_FORMAT),
+                    "target": target,
+                    "current_version": current_version,
+                    "target_version": target_version,
+                    "display_name": display_name,
+                    "release_notes": release_notes if release_notes else "",
+                    "success": success,
+                    "text": text,
+                },
+            )
+            self._update_log_dirty = True
+
+            if save:
+                self._save_update_log()
+
+    def _is_octoprint_outdated_and_can_update(self):
+        checks = self._get_configured_checks()
+        check = checks.get("octoprint", None)
+        if not check:
+            return False
+
+        check = self._populated_check("octoprint", check)
+        credentials = self._settings.get(["credentials"], merged=True)
+
+        _, update_available, update_possible, _, _ = self._get_current_version(
+            "octoprint", check, credentials=credentials
+        )
+        if not update_available or not update_possible:
+            return False
+
+        restart_type = self._get_restart_type(check)
+        restart_command = self._get_restart_command(restart_type)
+        return restart_command is not None
+
+    def _get_restart_type(self, check):
+        if check.get("restart") in self.VALID_RESTART_TYPES:
+            return check["restart"]
+        elif "pip" in check or check.get("method") in self.OCTOPRINT_RESTART_TYPES:
+            return "octoprint"
+        else:
+            target_restart_type = None
+
+        return target_restart_type
+
+    def _get_restart_command(self, restart_type):
+        if restart_type == "octoprint":
+            return self._settings.global_get(
+                ["server", "commands", "serverRestartCommand"]
+            )
+        elif restart_type == "environment":
+            return self._settings.global_get(
+                ["server", "commands", "systemRestartCommand"]
+            )
+        else:
+            return None
+
     # ~~ SettingsPlugin API
 
     def get_settings_defaults(self):
@@ -530,12 +724,14 @@ class SoftwareUpdatePlugin(
                 },
             },
             "pip_command": None,
-            "cache_ttl": 24 * 60,
+            "cache_ttl": 24 * 60,  # 1 day
             "notify_users": True,
             "ignore_throttled": False,
             "minimum_free_storage": 150,
             "check_overlay_url": "https://plugins.octoprint.org/update_check_overlay.json",
-            "check_overlay_ttl": 6 * 60,
+            "check_overlay_py2_url": "https://plugins.octoprint.org/update_check_overlay_py2.json",
+            "check_overlay_ttl": 6 * 60,  # 6 hours
+            "updatelog_cutoff": 30 * 24 * 60,  # 30 days
             "credentials": {
                 "github": None,
                 "bitbucket_user": None,
@@ -591,6 +787,7 @@ class SoftwareUpdatePlugin(
         else:
             data["octoprint_checkout_folder"] = None
             data["octoprint_type"] = None
+            data["octoprint_release_channel"] = None
 
         # ~~ pip check settings
 
@@ -907,10 +1104,43 @@ class SoftwareUpdatePlugin(
 
     # ~~ BluePrint API
 
+    @octoprint.plugin.BlueprintPlugin.route("/updatelog", methods=["GET"])
+    @no_firstrun_access
+    @Permissions.ADMIN.require(403)
+    def get_update_log(self):
+        def view():
+            return flask.jsonify(updatelog=list(reversed(self._update_log)))
+
+        def etag(lm=None):
+            hash = hashlib.sha1()
+
+            def hash_update(value):
+                value = value.encode("utf-8")
+                hash.update(value)
+
+            hash_update(repr(self._update_log))
+            return hash.hexdigest()
+
+        def lastmodified():
+            if os.path.exists(self._update_log_path):
+                return os.stat(self._update_log_path).st_mtime
+            else:
+                return 0
+
+        return with_revalidation_checking(
+            etag_factory=etag, lastmodified_factory=lastmodified
+        )(view)()
+
     @octoprint.plugin.BlueprintPlugin.route("/check", methods=["GET"])
     @no_firstrun_access
     @Permissions.PLUGIN_SOFTWAREUPDATE_CHECK.require(403)
     def check_for_update(self):
+        if (
+            not Permissions.PLUGIN_SOFTWAREUPDATE_CHECK.can()
+            and not self._settings.global_get(["server", "firstRun"])
+        ):
+            flask.abort(403)
+
         request_data = flask.request.values
 
         if "targets" in request_data or "check" in request_data:
@@ -1002,8 +1232,11 @@ class SoftwareUpdatePlugin(
                     }
                 )
             except exceptions.ConfigurationInvalid as e:
-                return flask.make_response(
-                    "Update not properly configured, can't proceed: {}".format(e), 500
+                flask.abort(
+                    500,
+                    description="Update not properly configured, can't proceed: {}".format(
+                        e
+                    ),
                 )
 
         def etag():
@@ -1057,9 +1290,13 @@ class SoftwareUpdatePlugin(
         )(view)()
 
     @octoprint.plugin.BlueprintPlugin.route("/update", methods=["POST"])
-    @no_firstrun_access
-    @Permissions.PLUGIN_SOFTWAREUPDATE_UPDATE.require(403)
     def perform_update(self):
+        if (
+            not Permissions.PLUGIN_SOFTWAREUPDATE_UPDATE.can()
+            and not self._settings.global_get(["server", "firstRun"])
+        ):
+            flask.abort(403)
+
         throttled = self._get_throttled()
         if (
             throttled
@@ -1068,30 +1305,35 @@ class SoftwareUpdatePlugin(
             and not self._settings.get_boolean(["ignore_throttled"])
         ):
             # currently throttled, we refuse to run
-            return flask.make_response(
+            message = (
                 "System is currently throttled, refusing to update "
-                "anything due to possible stability issues",
+                "anything due to possible stability issues"
+            )
+            self._logger.error(message)
+            flask.abort(
                 409,
+                description=message,
             )
 
         if self._printer.is_printing() or self._printer.is_paused():
             # do not update while a print job is running
-            return flask.make_response("Printer is currently printing or paused", 409)
+            flask.abort(409, description="Printer is currently printing or paused")
 
         if not self._environment_supported:
-            return flask.make_response(
-                "Direct updates are not supported in this Python environment", 409
+            flask.abort(
+                409,
+                description="Direct updates are not supported in this Python environment",
             )
 
         if not self._storage_sufficient:
-            return flask.make_response("Not enough free disk space for updating", 409)
+            flask.abort(409, description="Not enough free disk space for updating")
 
         if "application/json" not in flask.request.headers["Content-Type"]:
-            return flask.make_response("Expected content-type JSON", 400)
+            flask.abort(400, description="Expected content-type JSON")
 
         json_data = flask.request.get_json(silent=True)
         if json_data is None:
-            return flask.make_response("Invalid JSON", 400)
+            flask.abort(400, description="Invalid JSON")
 
         if "targets" in json_data or "checks" in json_data:
             targets = list(
@@ -1114,7 +1356,7 @@ class SoftwareUpdatePlugin(
     def configure_update(self):
         json_data = flask.request.get_json(silent=True)
         if json_data is None:
-            return flask.make_response("Invalid JSON", 400)
+            flask.abort(400)
 
         checks = self._get_configured_checks()
 
@@ -1205,20 +1447,61 @@ class SoftwareUpdatePlugin(
     def get_template_configs(self):
         from flask_babel import gettext
 
-        return [{"type": "settings", "name": gettext("Software Update")}]
+        templates = [
+            {"type": "settings", "name": gettext("Software Update")},
+        ]
+
+        if self._is_wizard_update_required():
+            templates.append(
+                {
+                    "type": "wizard",
+                    "name": gettext("Update?"),
+                    "template": "softwareupdate_wizard_update.jinja2",
+                    "suffix": "_update",
+                }
+            )
+
+        if self._is_settings_wizard_required():
+            templates.append(
+                {
+                    "type": "wizard",
+                    "name": gettext("Software Update"),
+                    "template": "softwareupdate_wizard_settings.jinja2",
+                    "suffix": "_settings",
+                }
+            )
+
+        return templates
 
     ##~~ WizardPlugin API
 
-    def is_wizard_required(self):
+    def _is_wizard_update_required(self):
+        firstrun = self._settings.global_get(["server", "firstRun"])
+        return (
+            firstrun
+            and self._is_octoprint_outdated_and_can_update()
+            and not self._is_settings_wizard_required()
+        )
+
+    def _is_settings_wizard_required(self):
         checks = self._get_configured_checks()
         check = checks.get("octoprint", None)
         checkout_folder = self._get_octoprint_checkout_folder(checks=checks)
-        return (
-            check
-            and "method" in check
-            and check["method"] == "update_script"
-            and not checkout_folder
-        )
+        return check and check.get("method") == "update_script" and not checkout_folder
+
+    def is_wizard_required(self):
+        return self._is_wizard_update_required() or self._is_settings_wizard_required()
+
+    def get_wizard_version(self):
+        return 1
+
+    def get_wizard_details(self):
+        result = {}
+        if self._is_wizard_update_required():
+            information, _, _ = self.get_current_versions()
+            data = information.get("octoprint", {})
+            result["update"] = data
+        return result
 
     ##~~ EventHandlerPlugin API
 
@@ -1253,6 +1536,8 @@ class SoftwareUpdatePlugin(
         if check_targets is None:
             check_targets = list(checks.keys())
 
+        # TODO: caching doesn't take check_targets into account!
+
         update_available = False
         update_possible = False
         information = {}
@@ -1336,6 +1621,7 @@ class SoftwareUpdatePlugin(
                         )
 
                         target_disabled = populated_check.get("disabled", False)
+                        target_compatible = populated_check.get("compatible", True)
                         update_available = update_available or (
                             target_update_available and not target_disabled
                         )
@@ -1348,28 +1634,9 @@ class SoftwareUpdatePlugin(
                         local_name = target_information["local"]["name"]
                         local_value = target_information["local"]["value"]
 
-                        release_notes = None
-                        if (
-                            target_information
-                            and target_information["remote"]
-                            and target_information["remote"]["value"]
-                        ):
-                            if (
-                                "release_notes" in populated_check
-                                and populated_check["release_notes"]
-                            ):
-                                release_notes = populated_check["release_notes"]
-                            elif "release_notes" in target_information["remote"]:
-                                release_notes = target_information["remote"][
-                                    "release_notes"
-                                ]
-
-                            if release_notes:
-                                release_notes = release_notes.format(
-                                    octoprint_version=VERSION,
-                                    target_name=target_information["remote"]["name"],
-                                    target_version=target_information["remote"]["value"],
-                                )
+                        release_notes = self._get_release_notes(
+                            target_information, populated_check
+                        )
 
                         information[target] = {
                             "updateAvailable": target_update_available,
@@ -1385,6 +1652,7 @@ class SoftwareUpdatePlugin(
                             "online": target_online,
                             "error": target_error,
                             "disabled": target_disabled,
+                            "compatible": target_compatible,
                             "releaseChannels": {},
                         }
 
@@ -1437,6 +1705,24 @@ class SoftwareUpdatePlugin(
 
         return information, update_available, update_possible
 
+    def _get_release_notes(self, information, populated_check):
+        release_notes = None
+
+        if information and information["remote"] and information["remote"]["value"]:
+            if populated_check.get("release_notes"):
+                release_notes = populated_check["release_notes"]
+            elif "release_notes" in information["remote"]:
+                release_notes = information["remote"]["release_notes"]
+
+            if release_notes:
+                release_notes = release_notes.format(
+                    octoprint_version=VERSION,
+                    target_name=information["remote"]["name"],
+                    target_version=information["remote"]["value"],
+                )
+
+        return release_notes
+
     def _get_check_hash(self, check):
         def dict_to_sorted_repr(d):
             lines = []
@@ -1490,6 +1776,9 @@ class SoftwareUpdatePlugin(
         update_available = False
         error = None
 
+        disabled = check.get("disabled", False)
+        compatible = check.get("compatible", True)
+
         try:
             version_checker = self._get_version_checker(target, check)
             information, is_current = version_checker.get_latest(
@@ -1506,7 +1795,7 @@ class SoftwareUpdatePlugin(
                     )
                     del self._version_cache[target]
                     self._version_cache_dirty = True
-                elif not is_current:
+                elif not is_current and compatible:
                     update_available = True
         except exceptions.CannotCheckOffline:
             update_possible = False
@@ -1544,15 +1833,15 @@ class SoftwareUpdatePlugin(
         else:
             try:
                 updater = self._get_updater(target, check)
-                update_possible = updater.can_perform_update(target, check, online=online)
+                update_possible = compatible and updater.can_perform_update(
+                    target, check, online=online
+                )
             except Exception:
                 self._logger.exception(
                     "Error while checking if {} can be updated".format(target)
                 )
                 update_possible = False
 
-        disabled = check.get("disabled", False)
-
         self._version_cache[target] = {
             "timestamp": time.time(),
             "hash": current_hash,
@@ -1562,6 +1851,7 @@ class SoftwareUpdatePlugin(
             "online": online,
             "error": error,
             "disabled": disabled,
+            "compatible": compatible,
         }
         self._version_cache_dirty = True
         return information, update_available, update_possible, online, error
@@ -1617,11 +1907,11 @@ class SoftwareUpdatePlugin(
         updater_thread.daemon = False
         updater_thread.start()
 
-        check_data = dict(
-            (key, check["displayName"] if "displayName" in check else key)
+        check_data = {
+            key: check["displayName"] if "displayName" in check else key
             for key, check in populated_checks.items()
             if key in to_be_updated
-        )
+        }
         return to_be_updated, check_data
 
     def _update_worker(self, checks, check_targets, force):
@@ -1643,7 +1933,7 @@ class SoftwareUpdatePlugin(
                     continue
                 check = checks[target]
 
-                if "enabled" in check and not check["enabled"]:
+                if not check.get("enabled", True) or not check.get("compatible", True):
                     continue
 
                 if target not in check_targets:
@@ -1656,15 +1946,7 @@ class SoftwareUpdatePlugin(
                 if target_result is not None:
                     target_results[target] = target_result
 
-                    if "restart" in check:
-                        target_restart_type = check["restart"]
-                    elif (
-                        "pip" in check
-                        or check.get("method") in self.OCTOPRINT_RESTART_TYPES
-                    ):
-                        target_restart_type = "octoprint"
-                    else:
-                        target_restart_type = None
+                    target_restart_type = self._get_restart_type(check)
 
                     # if our update requires a restart we have to determine which type
                     if restart_type is None or (
@@ -1677,6 +1959,9 @@ class SoftwareUpdatePlugin(
             # we might have needed to update the config, so we'll save that now
             self._settings.save()
 
+            # we also need to save our update log changes
+            self._save_update_log()
+
             # also, we are now longer updating
             self._update_in_progress = False
 
@@ -1692,16 +1977,7 @@ class SoftwareUpdatePlugin(
                 # one of our updates requires a restart of either type "octoprint" or "environment". Let's see if
                 # we can actually perform that
 
-                restart_command = None
-                if restart_type == "octoprint":
-                    restart_command = self._settings.global_get(
-                        ["server", "commands", "serverRestartCommand"]
-                    )
-                elif restart_type == "environment":
-                    restart_command = self._settings.global_get(
-                        ["server", "commands", "systemRestartCommand"]
-                    )
-
+                restart_command = self._get_restart_command(restart_type)
                 if restart_command:
                     self._send_client_message(
                         "restarting",
@@ -1730,15 +2006,40 @@ class SoftwareUpdatePlugin(
         information, update_available, update_possible, _, _ = self._get_current_version(
             target, check, online=online, credentials=credentials
         )
+        populated_check = self._populated_check(target, check)
+
+        def add_update_log(success, text):
+            self._add_update_log_entry(
+                target,
+                information["local"]["value"],
+                information["remote"]["value"],
+                success,
+                text,
+                display_name=populated_check["displayName"],
+                release_notes=self._get_release_notes(information, populated_check),
+                save=False,
+            )
 
         if not update_available and not force:
+            add_update_log(False, "Target is already up to date")
             return False, None
 
         if not update_possible:
             self._logger.warning(
-                "Cannot perform update for %s, update type is not fully configured"
-                % target
+                "Cannot perform update for {}, update type is not fully configured".format(
+                    target
+                )
             )
+            add_update_log(False, "Update not possible, missing information")
+            return False, None
+
+        if not information.get("compatible", True):
+            self._logger.warning(
+                "Cannot perform update for {}, update is reported as incompatible".format(
+                    target
+                )
+            )
+            add_update_log(False, "Update is incompatible")
             return False, None
 
         # determine the target version to update to
@@ -1769,9 +2070,10 @@ class SoftwareUpdatePlugin(
 
         ### The actual update procedure starts here...
 
-        populated_check = self._populated_check(target, check)
         try:
-            self._logger.info("Starting update of %s to %s..." % (target, target_version))
+            self._logger.info(
+                "Starting update of {} to {}...".format(target, target_version)
+            )
             self._send_client_message(
                 "updating",
                 {
@@ -1788,7 +2090,9 @@ class SoftwareUpdatePlugin(
                 target, populated_check, target_version, log_cb=self._log, online=online
             )
             target_result = ("success", update_result)
-            self._logger.info("Update of %s to %s successful!" % (target, target_version))
+            self._logger.info(
+                "Update of {} to {} successful!".format(target, target_version)
+            )
             trigger_event(True)
 
         except exceptions.UnknownUpdateType:
@@ -1804,6 +2108,7 @@ class SoftwareUpdatePlugin(
                     "reason": "Unknown update type",
                 },
             )
+            add_update_log(False, "Update failed, unknown update type")
             return False, None
 
         except exceptions.CannotUpdateOffline:
@@ -1820,6 +2125,8 @@ class SoftwareUpdatePlugin(
                     "reason": "No internet connection",
                 },
             )
+            add_update_log(False, "No internet connection")
+            return False, None
 
         except Exception as e:
             self._logger.exception(
@@ -1853,6 +2160,7 @@ class SoftwareUpdatePlugin(
                         "reason": "unknown",
                     },
                 )
+            add_update_log(False, "Update failed")
 
         else:
             # make sure that any external changes to config.yaml are loaded into the system
@@ -1864,6 +2172,8 @@ class SoftwareUpdatePlugin(
             del self._version_cache[target]
             self._version_cache_dirty = True
 
+            add_update_log(True, "Update successful")
+
         return target_error, target_result
 
     def _persist_check_current(self, target, check, current):
@@ -1972,12 +2282,6 @@ class SoftwareUpdatePlugin(
                     elif check.get("pip", None):
                         # we force python unequality check for pip installs, to be able to downgrade
                         result["release_compare"] = "python_unequal"
-                        result["pip_command"] = check.get(
-                            "pip_command",
-                            self._settings.global_get(
-                                ["server", "commands", "localPipCommand"]
-                            ),
-                        )
 
         elif target == "pip":
             import pkg_resources
@@ -2021,12 +2325,13 @@ class SoftwareUpdatePlugin(
                     "current", check.get("displayVersion", None)
                 )
 
-        if "pip" in result:
-            if (
-                "pip_command" not in check
-                and self._settings.get(["pip_command"]) is not None
-            ):
-                result["pip_command"] = self._settings.get(["pip_command"])
+        if result.get("pip", None):
+            if "pip_command" not in result:
+                local_pip_command = self._settings.global_get(
+                    ["server", "commands", "localPipCommand"]
+                )
+                if local_pip_command:
+                    result["pip_command"] = local_pip_command
 
         return result
 
@@ -2172,4 +2477,5 @@ def __plugin_load__():
         "octoprint.cli.commands": cli.commands,
         "octoprint.events.register_custom_events": _register_custom_events,
         "octoprint.access.permissions": __plugin_implementation__.get_additional_permissions,
+        "octoprint.systeminfo.additional_bundle_files": __plugin_implementation__.get_additional_bundle_files,
     }
