@@ -38,6 +38,7 @@ class TrackingPlugin(
     octoprint.plugin.AssetPlugin,
     octoprint.plugin.WizardPlugin,
     octoprint.plugin.EventHandlerPlugin,
+    octoprint.plugin.SimpleApiPlugin,
 ):
     def __init__(self):
         self._environment = None
@@ -76,6 +77,7 @@ class TrackingPlugin(
                 "printer_safety_check": True,
                 "throttled": True,
                 "slicing": True,
+                "webui_load": True,
             },
         }
 
@@ -94,6 +96,26 @@ class TrackingPlugin(
             # tracking was just enabled, let's start up tracking
             self._start_tracking()
 
+    ##~~ SimpleApiPlugin
+
+    def get_api_commands(self):
+        return {"track": ["event", "payload"]}
+
+    def on_api_command(self, command, data):
+        if command != "track":
+            return
+
+        event = data.pop("event")
+        if event == "webui_load":
+            payload = data.get("payload", {})
+            browser_name = payload.get("browser_name")
+            browser_version = payload.get("browser_version")
+            os_name = payload.get("os_name")
+            os_version = payload.get("os_version")
+
+            if all((browser_name, browser_version, os_name, os_version)):
+                self._track_webui_load(browser_name, browser_version, os_name, os_version)
+
     ##~~ EnvironmentDetectionPlugin
 
     def on_environment_detected(self, environment, *args, **kwargs):
@@ -191,7 +213,7 @@ class TrackingPlugin(
     ##~~ AssetPlugin
 
     def get_assets(self):
-        return {"js": ["js/usage.js"]}
+        return {"js": ["js/usage.js"], "clientjs": ["clientjs/usage.js"]}
 
     ##~~ WizardPlugin
 
@@ -241,7 +263,8 @@ class TrackingPlugin(
             return
 
         uptime = int(monotonic_time() - self._startup_time)
-        self._track("ping", octoprint_uptime=uptime)
+        printer_state = self._printer.get_state_id()
+        self._track("ping", octoprint_uptime=uptime, printer_state=printer_state)
 
     def _track_pong(self):
         if not self._settings.get_boolean(["events", "pong"]):
@@ -279,6 +302,18 @@ class TrackingPlugin(
 
         self._track("shutdown")
 
+    def _track_webui_load(self, browser_name, browser_version, os_name, os_version):
+        if not self._settings.get_boolean(["events", "webui_load"]):
+            return
+
+        self._track(
+            "webui_load",
+            browser_name=browser_name,
+            browser_version=browser_version,
+            os_name=os_name,
+            os_version=os_version,
+        )
+
     def _track_plugin_event(self, event, payload):
         if not self._settings.get_boolean(["events", "plugin"]):
             return
