@@ -1,6 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 """
 This module contains printer profile related code.
 
@@ -141,6 +139,7 @@ A printer profile is a ``dict`` of the following structure:
 
 .. autoclass:: InvalidProfileError
 """
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
@@ -556,7 +555,7 @@ class PrinterProfileManager(object):
             self._logger.exception(
                 "Error while trying to save profile %s" % profile["id"]
             )
-            raise SaveError("Cannot save profile %s: %s" % (profile["id"], str(e)))
+            raise SaveError("Cannot save profile {}: {}".format(profile["id"], str(e)))
 
     def _remove_from_path(self, path):
         try:
@@ -796,8 +795,8 @@ class PrinterProfileManager(object):
     @staticmethod
     def _default_box_for_volume(volume):
         if volume["origin"] == BedOrigin.CENTER:
-            half_width = volume["width"] / 2.0
-            half_depth = volume["depth"] / 2.0
+            half_width = volume["width"] / 2
+            half_depth = volume["depth"] / 2
             return {
                 "x_min": -half_width,
                 "x_max": half_width,
