@@ -330,7 +330,7 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.SDCARD,
-                            filename=urlquote(f["name"]),
+                            filename=f["name"],
                             _external=True,
                         )
                     },
@@ -392,7 +392,7 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.LOCAL,
-                            filename=urlquote(path + file_or_folder["name"]),
+                            filename=path + file_or_folder["name"],
                             _external=True,
                         )
                     }
@@ -446,7 +446,7 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.LOCAL,
-                            filename=urlquote(file_or_folder["path"]),
+                            filename=file_or_folder["path"],
                             _external=True,
                         ),
                         "download": url_for("index", _external=True)
@@ -692,7 +692,7 @@ def uploadGcodeFile(target):
         location = url_for(
             ".readGcodeFile",
             target=FileDestinations.LOCAL,
-            filename=urlquote(filename),
+            filename=filename,
             _external=True,
         )
         files.update(
@@ -717,7 +717,7 @@ def uploadGcodeFile(target):
             location = url_for(
                 ".readGcodeFile",
                 target=FileDestinations.SDCARD,
-                filename=urlquote(sdFilename),
+                filename=sdFilename,
                 _external=True,
             )
             files.update(
@@ -769,7 +769,7 @@ def uploadGcodeFile(target):
         location = url_for(
             ".readGcodeFile",
             target=FileDestinations.LOCAL,
-            filename=urlquote(added_folder),
+            filename=added_folder,
             _external=True,
         )
         folder = {
@@ -1041,7 +1041,7 @@ def gcodeFileCommand(filename, target):
             location = url_for(
                 ".readGcodeFile",
                 target=target,
-                filename=urlquote(full_path),
+                filename=full_path,
                 _external=True,
             )
             result = {
@@ -1164,7 +1164,7 @@ def gcodeFileCommand(filename, target):
             location = url_for(
                 ".readGcodeFile",
                 target=target,
-                filename=urlquote(destination),
+                filename=destination,
                 _external=True,
             )
             result = {
