@@ -6,10 +6,11 @@ __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agp
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
 import logging
+import re
 
-from flask import abort, jsonify, make_response, request
+from flask import abort, jsonify, request
 from flask_login import current_user
-from werkzeug.exceptions import BadRequest
+from past.builtins import basestring
 
 import octoprint.plugin
 import octoprint.util
@@ -23,6 +24,7 @@ from octoprint.settings import settings, valid_boolean_trues
 
 FOLDER_TYPES = ("uploads", "timelapse", "timelapse_tmp", "logs", "watched")
 FOLDER_MAPPING = {"timelapseTmp": "timelapse_tmp"}
+TIMELAPSE_BITRATE_PATTERN = re.compile(r"\d+[KMGTPEZY]?i?B?", flags=re.IGNORECASE)
 
 
 def _lastmodified():
@@ -131,6 +133,7 @@ def getSettings():
             "snapshotTimeout": s.getInt(["webcam", "snapshotTimeout"]),
             "snapshotSslValidation": s.getBoolean(["webcam", "snapshotSslValidation"]),
             "ffmpegPath": s.get(["webcam", "ffmpeg"]),
+            "ffmpegCommandline": s.get(["webcam", "ffmpegCommandline"]),
             "bitrate": s.get(["webcam", "bitrate"]),
             "ffmpegThreads": s.get(["webcam", "ffmpegThreads"]),
             "ffmpegVideoCodec": s.get(["webcam", "ffmpegVideoCodec"]),
@@ -138,6 +141,7 @@ def getSettings():
             "flipH": s.getBoolean(["webcam", "flipH"]),
             "flipV": s.getBoolean(["webcam", "flipV"]),
             "rotate90": s.getBoolean(["webcam", "rotate90"]),
+            "cacheBuster": s.getBoolean(["webcam", "cacheBuster"]),
         },
         "feature": {
             "temperatureGraph": s.getBoolean(["feature", "temperatureGraph"]),
@@ -149,14 +153,21 @@ def getSettings():
             "printCancelConfirmation": s.getBoolean(
                 ["feature", "printCancelConfirmation"]
             ),
+            "uploadOverwriteConfirmation": s.getBoolean(
+                ["feature", "uploadOverwriteConfirmation"]
+            ),
             "g90InfluencesExtruder": s.getBoolean(["feature", "g90InfluencesExtruder"]),
             "autoUppercaseBlacklist": s.get(["feature", "autoUppercaseBlacklist"]),
         },
-        "gcodeAnalysis": {"runAt": s.get(["gcodeAnalysis", "runAt"])},
+        "gcodeAnalysis": {
+            "runAt": s.get(["gcodeAnalysis", "runAt"]),
+            "bedZ": s.getFloat(["gcodeAnalysis", "bedZ"]),
+        },
         "serial": {
             "port": connectionOptions["portPreference"],
             "baudrate": connectionOptions["baudratePreference"],
             "exclusive": s.getBoolean(["serial", "exclusive"]),
+            "lowLatency": s.getBoolean(["serial", "lowLatency"]),
             "portOptions": connectionOptions["ports"],
             "baudrateOptions": connectionOptions["baudrates"],
             "autoconnect": s.getBoolean(["serial", "autoconnect"]),
@@ -180,6 +191,7 @@ def getSettings():
             "timeoutSdStatusAutoreport": s.getFloat(
                 ["serial", "timeout", "sdStatusAutoreport"]
             ),
+            "timeoutPosAutoreport": s.getFloat(["serial", "timeout", "posAutoreport"]),
             "timeoutBaudrateDetectionPause": s.getFloat(
                 ["serial", "timeout", "baudrateDetectionPause"]
             ),
@@ -194,6 +206,7 @@ def getSettings():
             "longRunningCommands": s.get(["serial", "longRunningCommands"]),
             "checksumRequiringCommands": s.get(["serial", "checksumRequiringCommands"]),
             "blockedCommands": s.get(["serial", "blockedCommands"]),
+            "ignoredCommands": s.get(["serial", "ignoredCommands"]),
             "pausingCommands": s.get(["serial", "pausingCommands"]),
             "emergencyCommands": s.get(["serial", "emergencyCommands"]),
             "helloCommand": s.get(["serial", "helloCommand"]),
@@ -211,6 +224,7 @@ def getSettings():
             "neverSendChecksum": s.getBoolean(["serial", "neverSendChecksum"]),
             "sdRelativePath": s.getBoolean(["serial", "sdRelativePath"]),
             "sdAlwaysAvailable": s.getBoolean(["serial", "sdAlwaysAvailable"]),
+            "sdLowerCase": s.getBoolean(["serial", "sdLowerCase"]),
             "swallowOkAfterResend": s.getBoolean(["serial", "swallowOkAfterResend"]),
             "repetierTargetTemp": s.getBoolean(["serial", "repetierTargetTemp"]),
             "externalHeatupDetection": s.getBoolean(
@@ -238,10 +252,14 @@ def getSettings():
             "capAutoreportSdStatus": s.getBoolean(
                 ["serial", "capabilities", "autoreport_sdstatus"]
             ),
+            "capAutoreportPos": s.getBoolean(
+                ["serial", "capabilities", "autoreport_pos"]
+            ),
             "capBusyProtocol": s.getBoolean(["serial", "capabilities", "busy_protocol"]),
             "capEmergencyParser": s.getBoolean(
                 ["serial", "capabilities", "emergency_parser"]
             ),
+            "capExtendedM20": s.getBoolean(["serial", "capabilities", "extended_m20"]),
             "resendRatioThreshold": s.getInt(["serial", "resendRatioThreshold"]),
             "resendRatioStart": s.getInt(["serial", "resendRatioStart"]),
         },
@@ -310,6 +328,7 @@ def getSettings():
             "allowFraming": s.getBoolean(["server", "allowFraming"]),
         },
         "devel": {"pluginTimings": s.getBoolean(["devel", "pluginTimings"])},
+        "slicing": {"defaultSlicer": s.get(["slicing", "defaultSlicer"])},
     }
 
     gcode_scripts = s.listScripts("gcode")
@@ -368,22 +387,11 @@ def _get_plugin_settings():
 @Permissions.SETTINGS.require(403)
 def setSettings():
     if "application/json" not in request.headers["Content-Type"]:
-        return make_response("Expected content-type JSON", 400)
+        abort(400, description="Expected content-type JSON")
 
-    try:
-        data = request.get_json()
-    except BadRequest:
-        return make_response("Malformed JSON body in request", 400)
-
-    if data is None:
-        return make_response("Malformed JSON body in request", 400)
-
-    if not isinstance(data, dict):
-        return make_response(
-            "Malformed request, need settings dictionary, "
-            "got a {} instead: {!r}".format(type(data).__name__, data),
-            400,
-        )
+    data = request.get_json()
+    if data is None or not isinstance(data, dict):
+        abort(400, description="Malformed JSON body in request")
 
     response = _saveSettings(data)
     if response:
@@ -453,10 +461,10 @@ def _saveSettings(data):
 
     if "folder" in data:
         try:
-            folders = dict(
-                (FOLDER_MAPPING.get(folder, folder), path)
+            folders = {
+                FOLDER_MAPPING.get(folder, folder): path
                 for folder, path in data["folder"].items()
-            )
+            }
             future = {}
             for folder in FOLDER_TYPES:
                 future[folder] = s.getBaseFolder(folder)
@@ -479,7 +487,7 @@ def _saveSettings(data):
 
                 s.setBaseFolder(folder, future[folder])
         except Exception:
-            return make_response("At least one of the configured folders is invalid", 400)
+            abort(400, description="At least one of the configured folders is invalid")
 
     if "api" in data:
         if "allowCrossOrigin" in data["api"]:
@@ -552,8 +560,41 @@ def _saveSettings(data):
             )
         if "ffmpegPath" in data["webcam"]:
             s.set(["webcam", "ffmpeg"], data["webcam"]["ffmpegPath"])
-        if "bitrate" in data["webcam"]:
-            s.set(["webcam", "bitrate"], data["webcam"]["bitrate"])
+        if "ffmpegCommandline" in data["webcam"]:
+            commandline = data["webcam"]["ffmpegCommandline"]
+            if not all(
+                map(lambda x: "{" + x + "}" in commandline, ("ffmpeg", "input", "output"))
+            ):
+                abort(
+                    400,
+                    description="Invalid webcam.ffmpegCommandline setting, lacks mandatory {ffmpeg}, {input} or {output}",
+                )
+            try:
+                commandline.format(
+                    ffmpeg="ffmpeg",
+                    fps="fps",
+                    bitrate="bitrate",
+                    threads="threads",
+                    input="input",
+                    output="output",
+                    videocodec="videocodec",
+                    containerformat="containerformat",
+                    filters="filters",
+                )
+            except Exception:
+                # some invalid data we'll refuse to set
+                logger.exception("Invalid webcam.ffmpegCommandline setting")
+                abort(400, description="Invalid webcam.ffmpegCommandline setting")
+            else:
+                s.set(["webcam", "ffmpegCommandline"], commandline)
+        if "bitrate" in data["webcam"] and data["webcam"]["bitrate"]:
+            bitrate = str(data["webcam"]["bitrate"])
+            if not TIMELAPSE_BITRATE_PATTERN.match(bitrate):
+                abort(
+                    400,
+                    description="Invalid webcam.bitrate setting, needs to be a valid ffmpeg bitrate",
+                )
+            s.set(["webcam", "bitrate"], bitrate)
         if "ffmpegThreads" in data["webcam"]:
             s.setInt(["webcam", "ffmpegThreads"], data["webcam"]["ffmpegThreads"])
         if "ffmpegVideoCodec" in data["webcam"] and data["webcam"][
@@ -568,6 +609,8 @@ def _saveSettings(data):
             s.setBoolean(["webcam", "flipV"], data["webcam"]["flipV"])
         if "rotate90" in data["webcam"]:
             s.setBoolean(["webcam", "rotate90"], data["webcam"]["rotate90"])
+        if "cacheBuster" in data["webcam"]:
+            s.setBoolean(["webcam", "cacheBuster"], data["webcam"]["cacheBuster"])
 
     if "feature" in data:
         if "temperatureGraph" in data["feature"]:
@@ -596,6 +639,11 @@ def _saveSettings(data):
                 ["feature", "printCancelConfirmation"],
                 data["feature"]["printCancelConfirmation"],
             )
+        if "uploadOverwriteConfirmation" in data["feature"]:
+            s.setBoolean(
+                ["feature", "uploadOverwriteConfirmation"],
+                data["feature"]["uploadOverwriteConfirmation"],
+            )
         if "g90InfluencesExtruder" in data["feature"]:
             s.setBoolean(
                 ["feature", "g90InfluencesExtruder"],
@@ -612,6 +660,8 @@ def _saveSettings(data):
     if "gcodeAnalysis" in data:
         if "runAt" in data["gcodeAnalysis"]:
             s.set(["gcodeAnalysis", "runAt"], data["gcodeAnalysis"]["runAt"])
+        if "bedZ" in data["gcodeAnalysis"]:
+            s.setFloat(["gcodeAnalysis", "bedZ"], data["gcodeAnalysis"]["bedZ"])
 
     if "serial" in data:
         if "autoconnect" in data["serial"]:
@@ -622,6 +672,8 @@ def _saveSettings(data):
             s.setInt(["serial", "baudrate"], data["serial"]["baudrate"])
         if "exclusive" in data["serial"]:
             s.setBoolean(["serial", "exclusive"], data["serial"]["exclusive"])
+        if "lowLatency" in data["serial"]:
+            s.setBoolean(["serial", "lowLatency"], data["serial"]["lowLatency"])
         if "timeoutConnection" in data["serial"]:
             s.setFloat(
                 ["serial", "timeout", "connection"],
@@ -682,6 +734,12 @@ def _saveSettings(data):
                 data["serial"]["timeoutSdStatusAutoreport"],
                 min=0.0,
             )
+        if "timeoutPosAutoreport" in data["serial"]:
+            s.setFloat(
+                ["serial", "timeout", "posAutoreport"],
+                data["serial"]["timeoutPosAutoreport"],
+                min=0.0,
+            )
         if "timeoutBaudrateDetectionPause" in data["serial"]:
             s.setFloat(
                 ["serial", "timeout", "baudrateDetectionPause"],
@@ -731,6 +789,10 @@ def _saveSettings(data):
             data["serial"]["blockedCommands"], (list, tuple)
         ):
             s.set(["serial", "blockedCommands"], data["serial"]["blockedCommands"])
+        if "ignoredCommands" in data["serial"] and isinstance(
+            data["serial"]["ignoredCommands"], (list, tuple)
+        ):
+            s.set(["serial", "ignoredCommands"], data["serial"]["ignoredCommands"])
         if "pausingCommands" in data["serial"] and isinstance(
             data["serial"]["pausingCommands"], (list, tuple)
         ):
@@ -774,6 +836,8 @@ def _saveSettings(data):
             s.setBoolean(
                 ["serial", "sdAlwaysAvailable"], data["serial"]["sdAlwaysAvailable"]
             )
+        if "sdLowerCase" in data["serial"]:
+            s.setBoolean(["serial", "sdLowerCase"], data["serial"]["sdLowerCase"])
         if "swallowOkAfterResend" in data["serial"]:
             s.setBoolean(
                 ["serial", "swallowOkAfterResend"], data["serial"]["swallowOkAfterResend"]
@@ -858,6 +922,11 @@ def _saveSettings(data):
                 ["serial", "capabilities", "autoreport_sdstatus"],
                 data["serial"]["capAutoreportSdStatus"],
             )
+        if "capAutoreportPos" in data["serial"]:
+            s.setBoolean(
+                ["serial", "capabilities", "autoreport_pos"],
+                data["serial"]["capAutoreportPos"],
+            )
         if "capBusyProtocol" in data["serial"]:
             s.setBoolean(
                 ["serial", "capabilities", "busy_protocol"],
@@ -868,6 +937,11 @@ def _saveSettings(data):
                 ["serial", "capabilities", "emergency_parser"],
                 data["serial"]["capEmergencyParser"],
             )
+        if "capExtendedM20" in data["serial"]:
+            s.setBoolean(
+                ["serial", "capabilities", "extended_m20"],
+                data["serial"]["capExtendedM20"],
+            ),
         if "resendRatioThreshold" in data["serial"]:
             s.setInt(
                 ["serial", "resendRatioThreshold"], data["serial"]["resendRatioThreshold"]
@@ -932,6 +1006,8 @@ def _saveSettings(data):
             for name, script in data["scripts"]["gcode"].items():
                 if name == "snippets":
                     continue
+                if not isinstance(script, basestring):
+                    continue
                 s.saveScript(
                     "gcode", name, script.replace("\r\n", "\n").replace("\r", "\n")
                 )
@@ -1024,6 +1100,10 @@ def _saveSettings(data):
             logging.getLogger("PLUGIN_TIMINGS").setLevel(logging.DEBUG)
             logging.getLogger("PLUGIN_TIMINGS").debug("Enabling plugin timings logging")
 
+    if "slicing" in data:
+        if "defaultSlicer" in data["slicing"]:
+            s.set(["slicing", "defaultSlicer"], data["slicing"]["defaultSlicer"])
+
     if "plugins" in data:
         for plugin in octoprint.plugin.plugin_manager().get_implementations(
             octoprint.plugin.SettingsPlugin
