@@ -18,6 +18,7 @@ function DataUpdater(allViewModels, connectCallback, disconnectCallback) {
     self._lastProcessingTimesSize = 20;
 
     self._safeModePopup = undefined;
+    self._python2Popup = undefined;
 
     self.increaseThrottle = function () {
         self.setThrottle(self._throttleFactor + 1);
@@ -171,7 +172,13 @@ function DataUpdater(allViewModels, connectCallback, disconnectCallback) {
         VERSION = data["version"];
         DISPLAY_VERSION = data["display_version"];
         BRANCH = data["branch"];
+        PYTHON_VERSION = data["python_version"];
+
         $("span.version").text(DISPLAY_VERSION);
+        $("span.python_version").text(PYTHON_VERSION);
+
+        // update connectivity state
+        ONLINE = data["online"];
 
         // update plugin hash
         var oldPluginHash = self._pluginHash;
@@ -227,6 +234,22 @@ function DataUpdater(allViewModels, connectCallback, disconnectCallback) {
                 });
             }
 
+            // process python version
+            if (self._python2Popup) self._python2Popup.remove();
+            if (PYTHON_VERSION && PYTHON_VERSION.startsWith("2.")) {
+                self._python2Popup = new PNotify({
+                    title: gettext("You are still running Python 2"),
+                    text: _.sprintf(
+                        gettext(
+                            "<p>Python 2 is end-of-life as of January 1st 2020. While OctoPrint currently still supports running under Python 2, a future version will remove support and require Python 3. You should upgrade as soon as possible!</p><p>Please refer to the FAQ for recommended update workflows:</p>"
+                        ) +
+                            "<p><a href='https://faq.octoprint.org/python3-update' target='_blank' rel='noopener noreferer'>How to migrate to Python 3</a></p>",
+                        {reason: _.escape(reason)}
+                    ),
+                    hide: false
+                });
+            }
+
             // if the offline overlay is still showing, now's a good time to
             // hide it, plus reload the camera feed if it's currently displayed
             if ($("#offline_overlay").is(":visible")) {
@@ -413,6 +436,8 @@ function DataUpdater(allViewModels, connectCallback, disconnectCallback) {
                     type: severity,
                     hide: false
                 });
+            } else if (type === "ConnectivityChanged") {
+                ONLINE = payload.new;
             }
 
             var legacyEventHandlers = {
