@@ -755,7 +755,7 @@ function showMessageDialog(msg, options) {
 
     var modal = $("<div></div>").addClass("modal hide");
     if (!nofade) {
-        modal.addClass("fade");
+        modal.addClass("fade-in");
     }
     modal
         .append($("<div></div>").addClass("modal-header").append(modalHeader))
@@ -845,7 +845,7 @@ function showConfirmationDialog(msg, onacknowledge, options) {
 
     var modal = $("<div></div>").addClass("modal hide");
     if (!nofade) {
-        modal.addClass("fade");
+        modal.addClass("fade-in");
     }
 
     var buttons = $("<div></div>").addClass("modal-footer").append(cancelButton);
@@ -963,7 +963,7 @@ function showSelectionDialog(options) {
     // create modal and do final wiring up
     var modal = $("<div></div>").addClass("modal hide");
     if (!nofade) {
-        modal.addClass("fade");
+        modal.addClass("fade-in");
     }
     if (!cancel) {
         modal.data("backdrop", "static").data("keyboard", "false");
@@ -1060,18 +1060,14 @@ function showProgressModal(options, promise) {
 
     var progress = $('<div class="progress progress-text-centered"></div>');
     var progressBar = $('<div class="bar"></div>').addClass(barClassSuccess);
-    var progressTextBack = $('<span class="progress-text-back"></span>');
-    var progressTextFront = $('<span class="progress-text-front"></span>').width(
-        progress.width()
-    );
+    var progressText = $('<span class="progress-text-back"></span>');
 
     if (max === undefined) {
         progress.addClass("progress-striped active");
         progressBar.width("100%");
     }
 
-    progressBar.append(progressTextFront);
-    progress.append(progressTextBack).append(progressBar);
+    progress.append(progressBar).append(progressText);
 
     var button = $('<button class="btn">' + buttonText + "</button>")
         .prop("disabled", true)
@@ -1092,7 +1088,7 @@ function showProgressModal(options, promise) {
     }
 
     var modal = $("<div></div>")
-        .addClass("modal hide fade")
+        .addClass("modal hide fade-in")
         .addClass(dialogClass)
         .append($("<div></div>").addClass("modal-header").append(modalHeader))
         .append(modalBody)
@@ -1127,9 +1123,16 @@ function showProgressModal(options, promise) {
 
             // update progress bar
             progressBar.width(String(value) + "%");
-            progressTextFront.text(short);
-            progressTextBack.text(short);
-            progressTextFront.width(progress.width());
+            progressText.text(short);
+            if (value < 50 && progressText.hasClass("progress-text-front")) {
+                progressText
+                    .removeClass("progress-text-front")
+                    .addClass("progress-text-back");
+            } else if (value >= 50 && progressText.hasClass("progress-text-back")) {
+                progressText
+                    .removeClass("progress-text-back")
+                    .addClass("progress-text-front");
+            }
 
             // if not successful, apply failure class
             if (!success && !progressBar.hasClass(barClassFailure)) {
@@ -1592,3 +1595,27 @@ var loadFromLocalStorage = function (key) {
     }
     return current;
 };
+
+var deepMerge = function (target, source) {
+    /**
+     * Implements an object deep merge, which contrary to _.merge doesn't try to
+     * merge arrays.
+     */
+    if (!_.isObject(target)) {
+        return target;
+    }
+
+    _.forOwn(source, function (value, key) {
+        if (
+            target.hasOwnProperty(key) &&
+            _.isPlainObject(target[key]) &&
+            _.isPlainObject(value)
+        ) {
+            target[key] = deepMerge(target[key], value);
+        } else {
+            target[key] = value;
+        }
+    });
+
+    return target;
+};
