@@ -42,7 +42,12 @@ $(function () {
                 opera: false,
 
                 mobile: false,
-                desktop: false
+                desktop: false,
+
+                browser_name: "unknown",
+                browser_version: "unknown",
+                os_name: "unknown",
+                os_version: "unknown"
             },
             viewmodels: {},
             startedUp: false
@@ -118,6 +123,16 @@ $(function () {
         exports.browser.mobile = $.browser.mobile;
         exports.browser.desktop = !exports.browser.mobile;
 
+        var uap = UAParser();
+        exports.browser.browser_name = uap.browser.name;
+        exports.browser.browser_version = uap.browser.version;
+        exports.browser.os_name = uap.os.name;
+        exports.browser.os_version = uap.os.version;
+
+        if (exports.browser.safari) {
+            $("html").addClass("safari");
+        }
+
         // exports
 
         exports.isVisible = function () {
@@ -228,12 +243,15 @@ $(function () {
     [
         // printer states
         gettext("Offline"),
+        gettext("Offline after error"),
+        gettext("Error"),
         gettext("Opening serial connection"),
         gettext("Detecting serial connection"),
         gettext("Connecting"),
         gettext("Operational"),
         gettext("Starting"),
         gettext("Starting print from SD"),
+        gettext("Starting to send file to SD"),
         gettext("Printing from SD"),
         gettext("Sending file to SD"),
         gettext("Printing"),
