@@ -9,6 +9,8 @@ $(function () {
         self.printerProfiles = parameters[4];
         self.access = parameters[5];
 
+        self.allViewModels = undefined;
+
         self.filesListVisible = ko.observable(true);
         self.showInternalFilename = ko.observable(true);
 
@@ -83,6 +85,20 @@ $(function () {
         self.moveEntry = ko.observable({name: "", display: "", path: ""}); // is there a better way to do this?
         self.moveSource = ko.observable(undefined);
         self.moveDestination = ko.observable(undefined);
+        self.moveSourceFilename = ko.observable(undefined);
+        self.moveDestinationFilename = ko.observable(undefined);
+        self.moveDestinationFullpath = ko.pureComputed(function () {
+            // Join the paths for renaming
+            if (self.moveSourceFilename() != self.moveDestinationFilename()) {
+                if (self.moveDestination() === "/") {
+                    return self.moveDestination() + self.moveDestinationFilename();
+                } else {
+                    return self.moveDestination() + "/" + self.moveDestinationFilename();
+                }
+            } else {
+                return self.moveDestination();
+            }
+        });
         self.moveError = ko.observable("");
 
         self.folderList = ko.observableArray(["/"]);
@@ -97,9 +113,13 @@ $(function () {
             );
         });
 
+        self.uploadExistsDialog = undefined;
+        self.uploadFilename = ko.observable(undefined);
+
         self.allItems = ko.observable(undefined);
         self.currentPath = ko.observable("");
         self.uploadProgressText = ko.observable();
+        self.uploadProgressPercentage = ko.observable();
 
         // list style incl. persistence
         var listStyleStorageKey = "gcodeFiles.currentListStyle";
@@ -160,18 +180,22 @@ $(function () {
             listHelperExclusiveFilters.push(SUPPORTED_FILETYPES);
         }
 
+        var sortByName = function (a, b) {
+            // sorts ascending
+            if (a["display"].toLowerCase() < b["display"].toLowerCase()) return -1;
+            if (a["display"].toLowerCase() > b["display"].toLowerCase()) return 1;
+            return 0;
+        };
+
         self.listHelper = new ItemListHelper(
             "gcodeFiles",
             {
-                name: function (a, b) {
-                    // sorts ascending
-                    if (a["display"].toLowerCase() < b["display"].toLowerCase())
-                        return -1;
-                    if (a["display"].toLowerCase() > b["display"].toLowerCase()) return 1;
-                    return 0;
-                },
+                name: sortByName,
                 upload: function (a, b) {
                     // sorts descending
+                    if (a["date"] === undefined && b["date"] === undefined) {
+                        return sortByName(a, b);
+                    }
                     if (b["date"] === undefined || a["date"] > b["date"]) return -1;
                     if (a["date"] === undefined || a["date"] < b["date"]) return 1;
                     return 0;
@@ -607,13 +631,32 @@ $(function () {
                 return;
             }
 
+            var proceed = function (p) {
+                var prevented = false;
+                var callback = function () {
+                    OctoPrint.files.select(data.origin, data.path, p);
+                };
+
+                if (p) {
+                    callViewModels(self.allViewModels, "onBeforePrintStart", function (
+                        method
+                    ) {
+                        prevented = prevented || method(callback) === false;
+                    });
+                }
+
+                if (!prevented) {
+                    callback();
+                }
+            };
+
             if (
                 printAfterLoad &&
                 self.listHelper.isSelected(data) &&
                 self.enablePrint(data)
             ) {
                 // file was already selected, just start the print job
-                OctoPrint.job.start();
+                self.printerState.print();
             } else {
                 // select file, start print job (if requested and within dimensions)
                 var withinPrintDimensions = self.evaluatePrintDimensions(data, true);
@@ -628,12 +671,12 @@ $(function () {
                         cancel: gettext("No"),
                         proceed: gettext("Yes"),
                         onproceed: function () {
-                            OctoPrint.files.select(data.origin, data.path, print);
+                            proceed(print);
                         },
                         nofade: true
                     });
                 } else {
-                    OctoPrint.files.select(data.origin, data.path, print);
+                    proceed(print);
                 }
             }
         };
@@ -672,6 +715,8 @@ $(function () {
             self.moveError("");
             self.moveSource(current);
             self.moveDestination(current);
+            self.moveSourceFilename(entry.name);
+            self.moveDestinationFilename(entry.name);
             self.moveDialog.modal("show");
         };
 
@@ -920,60 +965,72 @@ $(function () {
         self.getAdditionalData = function (data) {
             var output = "";
             if (data["gcodeAnalysis"]) {
-                if (data["gcodeAnalysis"]["dimensions"]) {
-                    var dimensions = data["gcodeAnalysis"]["dimensions"];
-                    output +=
-                        gettext("Model size") +
-                        ": " +
-                        _.sprintf(
-                            "%(width).2fmm &times; %(depth).2fmm &times; %(height).2fmm",
-                            dimensions
-                        );
-                    output += "<br>";
-                }
                 if (
-                    data["gcodeAnalysis"]["filament"] &&
-                    typeof data["gcodeAnalysis"]["filament"] === "object"
+                    data["gcodeAnalysis"]["_empty"] ||
+                    !data["gcodeAnalysis"]["dimensions"] ||
+                    (data["gcodeAnalysis"]["dimensions"]["width"] === 0 &&
+                        data["gcodeAnalysis"]["dimensions"]["depth"] === 0 &&
+                        data["gcodeAnalysis"]["dimensions"]["height"] === 0)
                 ) {
-                    var filament = data["gcodeAnalysis"]["filament"];
-                    if (_.keys(filament).length === 1) {
+                    output += gettext("Model contains no extrusion.<br>");
+                } else {
+                    if (data["gcodeAnalysis"]["dimensions"]) {
+                        var dimensions = data["gcodeAnalysis"]["dimensions"];
                         output +=
-                            gettext("Filament") +
+                            gettext("Model size") +
                             ": " +
-                            formatFilament(
-                                data["gcodeAnalysis"]["filament"]["tool" + 0]
-                            ) +
-                            "<br>";
-                    } else if (_.keys(filament).length > 1) {
-                        _.each(filament, function (f, k) {
-                            if (
-                                !_.startsWith(k, "tool") ||
-                                !f ||
-                                !f.hasOwnProperty("length") ||
-                                f["length"] <= 0
-                            )
-                                return;
+                            _.sprintf(
+                                "%(width).2fmm &times; %(depth).2fmm &times; %(height).2fmm",
+                                dimensions
+                            );
+                        output += "<br>";
+                    }
+                    if (
+                        data["gcodeAnalysis"]["filament"] &&
+                        typeof data["gcodeAnalysis"]["filament"] === "object"
+                    ) {
+                        var filament = data["gcodeAnalysis"]["filament"];
+                        if (_.keys(filament).length === 1) {
                             output +=
                                 gettext("Filament") +
-                                " (" +
-                                gettext("Tool") +
-                                " " +
-                                k.substr("tool".length) +
-                                "): " +
-                                formatFilament(f) +
+                                ": " +
+                                formatFilament(
+                                    data["gcodeAnalysis"]["filament"]["tool" + 0]
+                                ) +
                                 "<br>";
-                        });
+                        } else if (_.keys(filament).length > 1) {
+                            _.each(filament, function (f, k) {
+                                if (
+                                    !_.startsWith(k, "tool") ||
+                                    !f ||
+                                    !f.hasOwnProperty("length") ||
+                                    f["length"] <= 0
+                                )
+                                    return;
+                                output +=
+                                    gettext("Filament") +
+                                    " (" +
+                                    gettext("Tool") +
+                                    " " +
+                                    k.substr("tool".length) +
+                                    "): " +
+                                    formatFilament(f) +
+                                    "<br>";
+                            });
+                        }
                     }
+                    output +=
+                        gettext("Estimated print time") +
+                        ": " +
+                        (self.settingsViewModel.appearance_fuzzyTimes()
+                            ? formatFuzzyPrintTime(
+                                  data["gcodeAnalysis"]["estimatedPrintTime"]
+                              )
+                            : formatDuration(
+                                  data["gcodeAnalysis"]["estimatedPrintTime"]
+                              )) +
+                        "<br>";
                 }
-                output +=
-                    gettext("Estimated print time") +
-                    ": " +
-                    (self.settingsViewModel.appearance_fuzzyTimes()
-                        ? formatFuzzyPrintTime(
-                              data["gcodeAnalysis"]["estimatedPrintTime"]
-                          )
-                        : formatDuration(data["gcodeAnalysis"]["estimatedPrintTime"])) +
-                    "<br>";
             }
             if (data["prints"] && data["prints"]["last"]) {
                 output +=
@@ -1130,7 +1187,10 @@ $(function () {
                         return false;
                     }
 
-                    var success = entry["name"].toLocaleLowerCase().indexOf(query) > -1;
+                    var success =
+                        (entry["display"] &&
+                            entry["display"].toLocaleLowerCase().indexOf(query) > -1) ||
+                        entry["name"].toLocaleLowerCase().indexOf(query) > -1;
                     if (!success && entry["type"] === "folder" && entry["children"]) {
                         return _.any(entry["children"], recursiveSearch);
                     }
@@ -1218,6 +1278,8 @@ $(function () {
                 }
             });
 
+            self.uploadExistsDialog = $("#upload_exists_dialog");
+
             //~~ Gcode upload
 
             self.uploadButton = $("#gcode_upload");
@@ -1282,6 +1344,7 @@ $(function () {
             self.uploadProgress.addClass("progress-striped").addClass("active");
             self.uploadProgressBar.css("width", "100%");
             if (payload.progressAvailable) {
+                self.uploadProgressPercentage(percentage);
                 self.uploadProgressText(
                     _.sprintf(gettext("Slicing ... (%(percentage)d%%)"), {percentage: 0})
                 );
@@ -1296,18 +1359,21 @@ $(function () {
                     percentage: Math.round(progress)
                 })
             );
+            self.uploadProgressPercentage(Math.round(progress));
         };
 
         self.onEventSlicingCancelled = function (payload) {
             self.uploadProgress.removeClass("progress-striped").removeClass("active");
             self.uploadProgressBar.css("width", "0%");
             self.uploadProgressText("");
+            self.uploadProgressPercentage(0);
         };
 
         self.onEventSlicingDone = function (payload) {
             self.uploadProgress.removeClass("progress-striped").removeClass("active");
             self.uploadProgressBar.css("width", "0%");
             self.uploadProgressText("");
+            self.uploadProgressPercentage(0);
 
             new PNotify({
                 title: gettext("Slicing done"),
@@ -1329,6 +1395,7 @@ $(function () {
             self.uploadProgress.removeClass("progress-striped").removeClass("active");
             self.uploadProgressBar.css("width", "0%");
             self.uploadProgressText("");
+            self.uploadProgressPercentage(0);
 
             var html = _.sprintf(
                 gettext("Could not slice %(stl)s to %(gcode)s: %(reason)s"),
@@ -1357,6 +1424,7 @@ $(function () {
         self.onEventTransferStarted = function (payload) {
             self.uploadProgress.addClass("progress-striped").addClass("active");
             self.uploadProgressBar.css("width", "100%");
+            self.uploadProgressPercentage(100);
             self.uploadProgressText(gettext("Streaming ..."));
         };
 
@@ -1364,6 +1432,7 @@ $(function () {
             self.uploadProgress.removeClass("progress-striped").removeClass("active");
             self.uploadProgressBar.css("width", "0");
             self.uploadProgressText("");
+            self.uploadProgressPercentage(0);
 
             new PNotify({
                 title: gettext("Streaming done"),
@@ -1387,6 +1456,7 @@ $(function () {
             self.uploadProgress.removeClass("progress-striped").removeClass("active");
             self.uploadProgressBar.css("width", "0");
             self.uploadProgressText("");
+            self.uploadProgressPercentage(0);
 
             new PNotify({
                 title: gettext("Streaming failed"),
@@ -1416,22 +1486,18 @@ $(function () {
 
             if (button === undefined) return;
 
-            button
-                .fileupload({
-                    url: url,
-                    dataType: "json",
-                    dropZone: enable ? drop : null,
-                    drop: function (e, data) {},
-                    submit: self._handleUploadStart,
-                    done: self._handleUploadDone,
-                    fail: self._handleUploadFail,
-                    always: self._handleUploadAlways,
-                    progressall: self._handleUploadProgress
-                })
-                .bind("fileuploadsubmit", function (e, data) {
-                    if (self.currentPath() !== "")
-                        data.formData = {path: self.currentPath()};
-                });
+            button.fileupload({
+                url: url,
+                dataType: "json",
+                dropZone: enable ? drop : null,
+                drop: function (e, data) {},
+                add: self._handleUploadAdd,
+                submit: self._handleUploadStart,
+                done: self._handleUploadDone,
+                fail: self._handleUploadFail,
+                always: self._handleUploadAlways,
+                progressall: self._handleUploadProgress
+            });
         };
 
         self._enableDragNDrop = function (enable) {
@@ -1449,6 +1515,7 @@ $(function () {
         self._setProgressBar = function (percentage, text, active) {
             self.uploadProgressBar.css("width", percentage + "%");
             self.uploadProgressText(text);
+            self.uploadProgressPercentage(percentage);
 
             if (active) {
                 self.uploadProgress.addClass("progress-striped active");
@@ -1457,6 +1524,89 @@ $(function () {
             }
         };
 
+        self._handleUploadAdd = function (e, data) {
+            var file = data.files[0];
+            var path = self.currentPath();
+
+            var formData = {};
+            if (path !== "") {
+                formData.path = path;
+            }
+
+            if (self.settingsViewModel.feature_uploadOverwriteConfirmation()) {
+                OctoPrint.files
+                    .exists("local", path, file.name)
+                    .done(function (response) {
+                        if (response.exists) {
+                            $("h3", self.uploadExistsDialog).text(
+                                _.sprintf(gettext("File already exists: %(name)s"), {
+                                    name: file.name
+                                })
+                            );
+                            $("input", self.uploadExistsDialog)
+                                .val("")
+                                .prop("placeholder", response.suggestion);
+                            $("a.upload-rename", self.uploadExistsDialog)
+                                .prop("disabled", false)
+                                .off("click")
+                                .on("click", function () {
+                                    var newName = $(
+                                        "input",
+                                        self.uploadExistsDialog
+                                    ).val();
+                                    if (newName === "") newName = response.suggestion;
+
+                                    OctoPrint.files
+                                        .exists("local", path, newName)
+                                        .done(function (r) {
+                                            if (r.exists) {
+                                                $(
+                                                    ".control-group",
+                                                    self.uploadExistsDialog
+                                                ).addClass("error");
+                                                $(
+                                                    ".help-block",
+                                                    self.uploadExistsDialog
+                                                ).show();
+                                            } else {
+                                                $(
+                                                    ".control-group",
+                                                    self.uploadExistsDialog
+                                                ).removeClass("error");
+                                                $(
+                                                    ".help-block",
+                                                    self.uploadExistsDialog
+                                                ).hide();
+
+                                                self.uploadExistsDialog.modal("hide");
+
+                                                formData.filename = newName;
+                                                formData.noOverwrite = true;
+                                                data.formData = formData;
+
+                                                data.submit();
+                                            }
+                                        });
+                                });
+                            $("a.upload-overwrite", self.uploadExistsDialog)
+                                .off("click")
+                                .on("click", function () {
+                                    self.uploadExistsDialog.modal("hide");
+                                    data.formData = formData;
+                                    data.submit();
+                                });
+                            self.uploadExistsDialog.modal("show");
+                        } else {
+                            data.formData = formData;
+                            data.submit();
+                        }
+                    });
+            } else {
+                data.formData = formData;
+                data.submit();
+            }
+        };
+
         self._handleUploadStart = function (e, data) {
             self.ignoreUpdatedFilesEvent = true;
             return true;
@@ -1590,6 +1740,9 @@ $(function () {
                 self.settingsViewModel.settings.appearance.showInternalFilename()
             );
         };
+        self.onAllBound = function (allViewModels) {
+            self.allViewModels = allViewModels;
+        };
     }
 
     OCTOPRINT_VIEWMODELS.push({
@@ -1604,6 +1757,11 @@ $(function () {
             "printerProfilesViewModel",
             "accessViewModel"
         ],
-        elements: ["#files_wrapper", "#add_folder_dialog", "#move_file_or_folder_dialog"]
+        elements: [
+            "#files_wrapper",
+            "#add_folder_dialog",
+            "#move_file_or_folder_dialog",
+            "#upload_exists_dialog"
+        ]
     });
 });
