@@ -157,6 +157,7 @@ $(function () {
         self.webcam_snapshotTimeout = ko.observable(undefined);
         self.webcam_snapshotSslValidation = ko.observable(undefined);
         self.webcam_ffmpegPath = ko.observable(undefined);
+        self.webcam_ffmpegCommandline = ko.observable(undefined);
         self.webcam_bitrate = ko.observable(undefined);
         self.webcam_ffmpegThreads = ko.observable(undefined);
         self.webcam_ffmpegVideoCodec = ko.observable(undefined);
@@ -164,6 +165,7 @@ $(function () {
         self.webcam_flipH = ko.observable(undefined);
         self.webcam_flipV = ko.observable(undefined);
         self.webcam_rotate90 = ko.observable(undefined);
+        self.webcam_cacheBuster = ko.observable(undefined);
 
         self.feature_temperatureGraph = ko.observable(undefined);
         self.feature_sdSupport = ko.observable(undefined);
@@ -172,6 +174,7 @@ $(function () {
         self.feature_modelSizeDetection = ko.observable(undefined);
         self.feature_printStartConfirmation = ko.observable(undefined);
         self.feature_printCancelConfirmation = ko.observable(undefined);
+        self.feature_uploadOverwriteConfirmation = ko.observable(undefined);
         self.feature_g90InfluencesExtruder = ko.observable(undefined);
         self.feature_autoUppercaseBlacklist = ko.observable(undefined);
 
@@ -180,6 +183,7 @@ $(function () {
         self.serial_port = ko.observable();
         self.serial_baudrate = ko.observable();
         self.serial_exclusive = ko.observable();
+        self.serial_lowLatency = ko.observable();
         self.serial_portOptions = ko.observableArray([]);
         self.serial_baudrateOptions = ko.observableArray([]);
         self.serial_autoconnect = ko.observable(undefined);
@@ -193,6 +197,7 @@ $(function () {
         self.serial_timeoutTemperatureAutoreport = ko.observable(undefined);
         self.serial_timeoutSdStatus = ko.observable(undefined);
         self.serial_timeoutSdStatusAutoreport = ko.observable(undefined);
+        self.serial_timeoutPosAutoreport = ko.observable(undefined);
         self.serial_timeoutBaudrateDetectionPause = ko.observable(undefined);
         self.serial_timeoutPositionLogWait = ko.observable(undefined);
         self.serial_log = ko.observable(undefined);
@@ -203,6 +208,7 @@ $(function () {
         self.serial_longRunningCommands = ko.observable(undefined);
         self.serial_checksumRequiringCommands = ko.observable(undefined);
         self.serial_blockedCommands = ko.observable(undefined);
+        self.serial_ignoredCommands = ko.observable(undefined);
         self.serial_pausingCommands = ko.observable(undefined);
         self.serial_emergencyCommands = ko.observable(undefined);
         self.serial_helloCommand = ko.observable(undefined);
@@ -211,6 +217,7 @@ $(function () {
         self.serial_waitForStart = ko.observable(undefined);
         self.serial_sendChecksum = ko.observable("print");
         self.serial_sdRelativePath = ko.observable(undefined);
+        self.serial_sdLowerCase = ko.observable(undefined);
         self.serial_sdAlwaysAvailable = ko.observable(undefined);
         self.serial_swallowOkAfterResend = ko.observable(undefined);
         self.serial_repetierTargetTemp = ko.observable(undefined);
@@ -229,8 +236,10 @@ $(function () {
         self.serial_maxTimeoutsLong = ko.observable(undefined);
         self.serial_capAutoreportTemp = ko.observable(undefined);
         self.serial_capAutoreportSdStatus = ko.observable(undefined);
+        self.serial_capAutoreportPos = ko.observable(undefined);
         self.serial_capBusyProtocol = ko.observable(undefined);
         self.serial_capEmergencyParser = ko.observable(undefined);
+        self.serial_capExtendedM20 = ko.observable(undefined);
         self.serial_sendM112OnError = ko.observable(undefined);
         self.serial_disableSdPrintingDetection = ko.observable(undefined);
         self.serial_ackMax = ko.observable(undefined);
@@ -406,7 +415,8 @@ $(function () {
         self.addTerminalFilter = function () {
             self.terminalFilters.push({
                 name: "New",
-                regex: "(Send: (Nd+s+)?M105)|(Recv:s+(oks+)?.*(B|Td*):d+)"
+                regex:
+                    "(Send:\\s+(N\\d+\\s+)?M105)|(Recv:\\s+(ok\\s+([PBN]\\d+\\s+)*)?.*([BCLPR]|T\\d*):-?\\d+)"
             });
         };
 
@@ -447,7 +457,10 @@ $(function () {
                 throw "Unknown stream type " + streamType;
             }
 
-            var message = $("<p></p>").append(text).append(webcam_element);
+            var message = $("<div id='webcamTestContainer'></div>")
+                .append($("<p></p>"))
+                .append(text)
+                .append(webcam_element);
 
             self.testWebcamStreamUrlBusy(true);
             showMessageDialog({
@@ -770,7 +783,7 @@ $(function () {
             $(".reload_nonconflicts", self.settingsUpdatedDialog).click(function (e) {
                 e.preventDefault();
                 self.settingsUpdatedDialog.modal("hide");
-                self.requestData(undefined, true);
+                self.requestData(true);
                 return false;
             });
 
@@ -798,7 +811,6 @@ $(function () {
                     }
                 })
                 .css({
-                    "width": "auto",
                     "margin-left": function () {
                         return -($(this).width() / 2);
                     }
@@ -839,14 +851,14 @@ $(function () {
         self.requestData = function (local) {
             // handle old parameter format
             var callback = undefined;
-            if (arguments.length == 2 || _.isFunction(local)) {
+            if (arguments.length === 2 || _.isFunction(local)) {
                 var exc = new Error();
                 log.warn(
                     "The callback parameter of SettingsViewModel.requestData is deprecated, the method now returns a promise, please use that instead. Stacktrace:",
                     exc.stack || exc.stacktrace || "<n/a>"
                 );
 
-                if (arguments.length == 2) {
+                if (arguments.length === 2) {
                     callback = arguments[0];
                     local = arguments[1];
                 } else {
@@ -1050,6 +1062,9 @@ $(function () {
                     blockedCommands: function () {
                         return splitTextToArray(self.serial_blockedCommands(), ",", true);
                     },
+                    ignoredCommands: function () {
+                        return splitTextToArray(self.serial_ignoredCommands(), ",", true);
+                    },
                     pausingCommands: function () {
                         return splitTextToArray(self.serial_pausingCommands(), ",", true);
                     },
@@ -1142,7 +1157,7 @@ $(function () {
                 // process all key-value-pairs here
                 _.forOwn(data, function (value, key) {
                     var observable = key;
-                    if (keyPrefix != undefined) {
+                    if (keyPrefix !== undefined) {
                         observable = keyPrefix + "_" + observable;
                     }
 
@@ -1161,7 +1176,7 @@ $(function () {
                             mapping && mapping[key] ? mapping[key] : undefined,
                             observable
                         );
-                        if (subresult != undefined) {
+                        if (subresult !== undefined) {
                             // we only set something on our result if we got something back
                             result[key] = subresult;
                             flag = true;
@@ -1179,7 +1194,7 @@ $(function () {
             // map local observables based on our existing data
             var dataFromObservables = mapFromObservables(data, specialMappings);
 
-            data = _.merge(data, dataFromObservables);
+            data = deepMerge(data, dataFromObservables);
             return data;
         };
 
@@ -1252,6 +1267,9 @@ $(function () {
                     blockedCommands: function (value) {
                         self.serial_blockedCommands(value.join(", "));
                     },
+                    ignoredCommands: function (value) {
+                        self.serial_ignoredCommands(value.join(", "));
+                    },
                     pausingCommands: function (value) {
                         self.serial_pausingCommands(value.join(", "));
                     },
